# Copyright (c) 2015 Hewlett-Packard Development Company, L.P.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

import base64
import socket
from ssl import SSLContext as sslc
import struct
import textwrap
from urllib import parse as urlparse

from oslo_log import log as logging
from oslo_utils import excutils

from tempest.common.utils.linux import remote_client
from tempest.common import waiters
from tempest import config
from tempest import exceptions
from tempest.lib.common import fixed_network
from tempest.lib.common import rest_client
from tempest.lib.common.utils import data_utils
from tempest.lib import exceptions as lib_exc

CONF = config.CONF

LOG = logging.getLogger(__name__)


def is_scheduler_filter_enabled(filter_name):
    """Check the list of enabled compute scheduler filters from config.

    This function checks whether the given compute scheduler filter is enabled
    in the nova config file. If the scheduler_enabled_filters option is set to
    'all' in tempest.conf then, this function returns True with assumption that
    requested filter 'filter_name' is one of the enabled filters in nova
    ("nova.scheduler.filters.all_filters").
    """

    filters = CONF.compute_feature_enabled.scheduler_enabled_filters
    if not filters:
        return False
    if 'all' in filters:
        return True
    if filter_name in filters:
        return True
    return False


def get_server_ip(server, validation_resources=None):
    """Get the server fixed or floating IP.

    Based on the configuration we're in, return a correct ip
    address for validating that a guest is up.

    :param server: The server dict as returned by the API
    :param validation_resources: The dict of validation resources
        provisioned for the server.
    """
    if CONF.validation.connect_method == 'floating':
        if validation_resources:
            return validation_resources['floating_ip']['ip']
        else:
            msg = ('When validation.connect_method equals floating, '
                   'validation_resources cannot be None')
            raise lib_exc.InvalidParam(invalid_param=msg)
    elif CONF.validation.connect_method == 'fixed':
        addresses = server['addresses'][CONF.validation.network_for_ssh]
        for address in addresses:
            if address['version'] == CONF.validation.ip_version_for_ssh:
                return address['addr']
        raise exceptions.ServerUnreachable(server_id=server['id'])
    else:
        raise lib_exc.InvalidConfiguration()


def _setup_validation_fip(
        server, clients, tenant_network, validation_resources):
    if CONF.service_available.neutron:
        ifaces = clients.interfaces_client.list_interfaces(server['id'])
        validation_port = None
        for iface in ifaces['interfaceAttachments']:
            if iface['net_id'] == tenant_network['id']:
                validation_port = iface['port_id']
                break
        if not validation_port:
            # NOTE(artom) This will get caught by the catch-all clause in
            # the wait_until loop below
            raise ValueError('Unable to setup floating IP for validation: '
                             'port not found on tenant network')
        clients.floating_ips_client.update_floatingip(
            validation_resources['floating_ip']['id'],
            port_id=validation_port)
    else:
        fip_client = clients.compute_floating_ips_client
        fip_client.associate_floating_ip_to_server(
            floating_ip=validation_resources['floating_ip']['ip'],
            server_id=server['id'])


def wait_for_ssh_or_ping(server, clients, tenant_network,
                         validatable, validation_resources, wait_until,
                         set_floatingip):
    """Wait for the server for SSH or Ping as requested.

    :param server: The server dict as returned by the API
    :param clients: Client manager which provides OpenStack Tempest clients.
    :param tenant_network: Tenant network to be used for creating a server.
    :param validatable: Whether the server will be pingable or sshable.
    :param validation_resources: Resources created for the connection to the
        server. Include a keypair, a security group and an IP.
    :param wait_until: Server status to wait for the server to reach.
        It can be PINGABLE and SSHABLE states when the server is both
        validatable and has the required validation_resources provided.
    :param set_floatingip: If FIP needs to be associated to server
    """
    if set_floatingip and CONF.validation.connect_method == 'floating':
        _setup_validation_fip(
            server, clients, tenant_network, validation_resources)

    server_ip = get_server_ip(
        server, validation_resources=validation_resources)
    if wait_until == 'PINGABLE':
        waiters.wait_for_ping(
            server_ip,
            clients.servers_client.build_timeout,
            clients.servers_client.build_interval
        )
    if wait_until == 'SSHABLE':
        pkey = validation_resources['keypair']['private_key']
        ssh_client = remote_client.RemoteClient(
            server_ip,
            CONF.validation.image_ssh_user,
            pkey=pkey,
            server=server,
            servers_client=clients.servers_client
        )
        waiters.wait_for_ssh(
            ssh_client,
            clients.servers_client.build_timeout
        )


def create_test_server(clients, validatable=False, validation_resources=None,
                       tenant_network=None, wait_until=None,
                       volume_backed=False, name=None, flavor=None,
                       image_id=None, **kwargs):
    """Common wrapper utility returning a test server.

    This method is a common wrapper returning a test server that can be
    pingable or sshable.

    :param clients: Client manager which provides OpenStack Tempest clients.
    :param validatable: Whether the server will be pingable or sshable.
    :param validation_resources: Resources created for the connection to the
        server. Include a keypair, a security group and an IP.
    :param tenant_network: Tenant network to be used for creating a server.
    :param wait_until: Server status to wait for the server to reach after
        its creation. Additionally PINGABLE and SSHABLE states are also
        accepted when the server is both validatable and has the required
        validation_resources provided.
    :param volume_backed: Whether the server is volume backed or not.
        If this is true, a volume will be created and create server will be
        requested with 'block_device_mapping_v2' populated with below values:

        .. code-block:: python

            bd_map_v2 = [{
                'uuid': volume['volume']['id'],
                'source_type': 'volume',
                'destination_type': 'volume',
                'boot_index': 0,
                'delete_on_termination': True}]
            kwargs['block_device_mapping_v2'] = bd_map_v2

        If server needs to be booted from volume with other combination of bdm
        inputs than mentioned above, then pass the bdm inputs explicitly as
        kwargs and image_id as empty string ('').
    :param name: Name of the server to be provisioned. If not defined a random
        string ending with '-instance' will be generated.
    :param flavor: Flavor of the server to be provisioned. If not defined,
        CONF.compute.flavor_ref will be used instead.
    :param image_id: ID of the image to be used to provision the server. If not
        defined, CONF.compute.image_ref will be used instead.
    :returns: a tuple
    """

    if name is None:
        name = data_utils.rand_name(
            prefix=CONF.resource_name_prefix, name=__name__ + "-instance")
    if flavor is None:
        flavor = CONF.compute.flavor_ref
    if image_id is None:
        image_id = CONF.compute.image_ref

    kwargs = fixed_network.set_networks_kwarg(
        tenant_network, kwargs) or {}

    multiple_create_request = (max(kwargs.get('min_count', 0),
                                   kwargs.get('max_count', 0)) > 1)

    if CONF.validation.run_validation and validatable:
        LOG.debug("Provisioning test server with validation resources %s",
                  validation_resources)
        if 'security_groups' in kwargs:
            kwargs['security_groups'].append(
                {'name': validation_resources['security_group']['name']})
        else:
            try:
                kwargs['security_groups'] = [
                    {'name': validation_resources['security_group']['name']}]
            except KeyError:
                LOG.debug("No security group provided.")

        if 'key_name' not in kwargs:
            try:
                kwargs['key_name'] = validation_resources['keypair']['name']
            except KeyError:
                LOG.debug("No key provided.")

        if CONF.validation.connect_method == 'floating':
            if wait_until is None:
                wait_until = 'ACTIVE'

        if 'user_data' not in kwargs:
            # If nothing overrides the default user data script then run
            # a simple script on the host to print networking info. This is
            # to aid in debugging ssh failures.
            script = '''
                     #!/bin/sh
                     echo "Printing {user} user authorized keys"
                     cat ~{user}/.ssh/authorized_keys || true
                     '''.format(user=CONF.validation.image_ssh_user)
            script_clean = textwrap.dedent(script).lstrip().encode('utf8')
            script_b64 = base64.b64encode(script_clean)
            kwargs['user_data'] = script_b64

    if volume_backed:
        volume_name = data_utils.rand_name(
            prefix=CONF.resource_name_prefix, name=__name__ + '-volume')
        volumes_client = clients.volumes_client_latest
        params = {'name': volume_name,
                  'imageRef': image_id,
                  'size': CONF.volume.volume_size}
        if CONF.compute.compute_volume_common_az:
            params.setdefault('availability_zone',
                              CONF.compute.compute_volume_common_az)
        volume = volumes_client.create_volume(**params)
        try:
            waiters.wait_for_volume_resource_status(volumes_client,
                                                    volume['volume']['id'],
                                                    'available')
        except Exception:
            with excutils.save_and_reraise_exception():
                try:
                    volumes_client.delete_volume(volume['volume']['id'])
                    volumes_client.wait_for_resource_deletion(
                        volume['volume']['id'])
                except Exception as exc:
                    LOG.exception("Deleting volume %s failed, exception %s",
                                  volume['volume']['id'], exc)
        bd_map_v2 = [{
            'uuid': volume['volume']['id'],
            'source_type': 'volume',
            'destination_type': 'volume',
            'boot_index': 0,
            'delete_on_termination': True}]
        kwargs['block_device_mapping_v2'] = bd_map_v2

        # Since this is boot from volume an image does not need
        # to be specified.
        image_id = ''

    if CONF.compute.compute_volume_common_az:
        kwargs.setdefault('availability_zone',
                          CONF.compute.compute_volume_common_az)
    body = clients.servers_client.create_server(name=name, imageRef=image_id,
                                                flavorRef=flavor,
                                                **kwargs)
    request_id = body.response['x-openstack-request-id']

    # handle the case of multiple servers
    if multiple_create_request:
        # Get servers created which name match with name param.
        body_servers = clients.servers_client.list_servers()
        created_servers = \
            [s for s in body_servers['servers'] if s['name'].startswith(name)]
    else:
        body = rest_client.ResponseBody(body.response, body['server'])
        created_servers = [body]

    if wait_until:

        # NOTE(lyarwood): PINGABLE and SSHABLE both require the instance to
        # go ACTIVE initially before we can setup the fip(s) etc so stash
        # this additional wait state for later use.
        wait_until_extra = None
        if wait_until in ['PINGABLE', 'SSHABLE']:
            if not validatable and validation_resources is None:
                raise RuntimeError(
                    'SSHABLE/PINGABLE requires validatable=True '
                    'and validation_resources to be passed')
            wait_until_extra = wait_until
            wait_until = 'ACTIVE'

        servers = []
        try:
            # Wait for server to be in active state and populate servers list
            # with those full server response so that we will have addresses
            # field present in server which is needed to be used for wait for
            # ssh
            for server in created_servers:
                server = waiters.wait_for_server_status(
                    clients.servers_client, server['id'], wait_until,
                    request_id=request_id)
                servers.append(server)

            for server in servers:
                if CONF.validation.run_validation and validatable:
                    if CONF.validation.connect_method == 'floating':
                        _setup_validation_fip(
                            server, clients, tenant_network,
                            validation_resources)
                    if wait_until_extra:
                        wait_for_ssh_or_ping(
                            server, clients, tenant_network,
                            validatable, validation_resources,
                            wait_until_extra, False)
        except Exception:
            with excutils.save_and_reraise_exception():
                for server in created_servers:
                    try:
                        clients.servers_client.delete_server(
                            server['id'])
                    except Exception:
                        LOG.exception('Deleting server %s failed',
                                      server['id'])
                for server in created_servers:
                    # NOTE(artom) If the servers were booted with volumes
                    # and with delete_on_termination=False we need to wait
                    # for the servers to go away before proceeding with
                    # cleanup, otherwise we'll attempt to delete the
                    # volumes while they're still attached to servers that
                    # are in the process of being deleted.
                    try:
                        waiters.wait_for_server_termination(
                            clients.servers_client, server['id'])
                    except Exception:
                        LOG.exception('Server %s failed to delete in time',
                                      server['id'])
        if servers and not multiple_create_request:
            body = rest_client.ResponseBody(body.response, servers[0])
        return body, servers

    return body, created_servers


def shelve_server(servers_client, server_id, force_shelve_offload=False):
    """Common wrapper utility to shelve server.

    This method is a common wrapper to make server in 'SHELVED'
    or 'SHELVED_OFFLOADED' state.

    :param servers_clients: Compute servers client instance.
    :param server_id: Server to make in shelve state
    :param force_shelve_offload: Forcefully offload shelve server if it
                                 is configured not to offload server
                                 automatically after offload time.
    """
    body = servers_client.shelve_server(server_id)
    request_id = body.response['x-openstack-request-id']

    offload_time = CONF.compute.shelved_offload_time
    if offload_time >= 0:
        waiters.wait_for_server_status(servers_client, server_id,
                                       'SHELVED_OFFLOADED',
                                       extra_timeout=offload_time,
                                       request_id=request_id)
    else:
        waiters.wait_for_server_status(servers_client, server_id, 'SHELVED')
        if force_shelve_offload:
            servers_client.shelve_offload_server(server_id)
            waiters.wait_for_server_status(servers_client, server_id,
                                           'SHELVED_OFFLOADED',
                                           request_id=request_id)


def create_websocket(url):
    url = urlparse.urlparse(url)

    # NOTE(mnaser): It is possible that there is no port specified, so fall
    #               back to the default port based on the scheme.
    port = url.port or (443 if url.scheme == 'https' else 80)

    for res in socket.getaddrinfo(url.hostname, port,
                                  socket.AF_UNSPEC, socket.SOCK_STREAM):
        af, socktype, proto, _, sa = res
        client_socket = socket.socket(af, socktype, proto)
        if url.scheme == 'https':
            client_socket = sslc().wrap_socket(client_socket,
                                               server_hostname=url.hostname)
        client_socket.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
        try:
            client_socket.connect(sa)
        except socket.error:
            client_socket.close()
            continue
        break
    else:
        raise socket.error('WebSocket creation failed')
    # Turn the Socket into a WebSocket to do the communication
    return _WebSocket(client_socket, url)


class _WebSocket(object):
    def __init__(self, client_socket, url):
        """Constructor for the WebSocket wrapper to the socket."""
        self._socket = client_socket
        # cached stream for early frames.
        self.cached_stream = b''
        # Upgrade the HTTP connection to a WebSocket
        self._upgrade(url)

    def _recv(self, recv_size):
        """Wrapper to receive data from the cached stream or socket."""
        if recv_size <= 0:
            return None

        data_from_cached = b''
        data_from_socket = b''
        if len(self.cached_stream) > 0:
            read_from_cached = min(len(self.cached_stream), recv_size)
            data_from_cached += self.cached_stream[:read_from_cached]
            self.cached_stream = self.cached_stream[read_from_cached:]
            recv_size -= read_from_cached
        if recv_size > 0:
            data_from_socket = self._socket.recv(recv_size)
        return data_from_cached + data_from_socket

    def receive_frame(self):
        """Wrapper for receiving data to parse the WebSocket frame format"""
        # We need to loop until we either get some bytes back in the frame
        # or no data was received (meaning the socket was closed).  This is
        # done to handle the case where we get back some empty frames
        while True:
            header = self._recv(2)
            # If we didn't receive any data, just return None
            if not header:
                return None
            # We will make the assumption that we are only dealing with
            # frames less than 125 bytes here (for the negotiation) and
            # that only the 2nd byte contains the length, and since the
            # server doesn't do masking, we can just read the data length
            if int(header[1]) & 127 > 0:
                return self._recv(int(header[1]) & 127)

    def send_frame(self, data):
        """Wrapper for sending data to add in the WebSocket frame format."""
        frame_bytes = list()
        # For the first byte, want to say we are sending binary data (130)
        frame_bytes.append(130)
        # Only sending negotiation data so don't need to worry about > 125
        # We do need to add the bit that says we are masking the data
        frame_bytes.append(len(data) | 128)
        # We don't really care about providing a random mask for security
        # So we will just hard-code a value since a test program
        mask = [7, 2, 1, 9]
        for i in range(len(mask)):
            frame_bytes.append(mask[i])
        # Mask each of the actual data bytes that we are going to send
        for i in range(len(data)):
            frame_bytes.append(int(data[i]) ^ mask[i % 4])
        # Convert our integer list to a binary array of bytes
        frame_bytes = struct.pack('!%iB' % len(frame_bytes), * frame_bytes)
        self._socket.sendall(frame_bytes)

    def close(self):
        """Helper method to close the connection."""
        # Close down the real socket connection and exit the test program
        if self._socket is not None:
            self._socket.shutdown(1)
            self._socket.close()
            self._socket = None

    def _upgrade(self, url):
        """Upgrade the HTTP connection to a WebSocket and verify."""
        # It is possible to pass the path as a query parameter in the request,
        # so use it if present
        # Given noVNC format
        # https://x.com/vnc_auto.html?path=%3Ftoken%3Dxxx,
        # url format is
        # ParseResult(scheme='https', netloc='x.com',
        # path='/vnc_auto.html', params='',
        # query='path=%3Ftoken%3Dxxx', fragment='').
        # qparams format is {'path': ['?token=xxx']}
        qparams = urlparse.parse_qs(url.query)
        # according to references
        # https://docs.python.org/3/library/urllib.parse.html
        # https://tools.ietf.org/html/rfc3986#section-3.4
        # qparams['path'][0] format is '?token=xxx' without / prefix
        # remove / in /websockify to comply to references.
        path = qparams['path'][0] if 'path' in qparams else 'websockify'
        # Fix websocket request format by adding / prefix.
        # Updated request format: GET /?token=xxx HTTP/1.1
        # or GET /websockify HTTP/1.1
        reqdata = 'GET /%s HTTP/1.1\r\n' % path
        reqdata += 'Host: %s' % url.hostname
        # Add port only if we have one specified
        if url.port:
            reqdata += ':%s' % url.port
        # Line-ending for Host header
        reqdata += '\r\n'
        # Tell the HTTP Server to Upgrade the connection to a WebSocket
        reqdata += 'Upgrade: websocket\r\nConnection: Upgrade\r\n'
        # The token=xxx is sent as a Cookie not in the URI for noVNC < v1.1.0
        reqdata += 'Cookie: %s\r\n' % url.query
        # Use a hard-coded WebSocket key since a test program
        reqdata += 'Sec-WebSocket-Key: x3JJHMbDL1EzLkh9GBhXDw==\r\n'
        reqdata += 'Sec-WebSocket-Version: 13\r\n'
        # We are choosing to use binary even though browser may do Base64
        reqdata += 'Sec-WebSocket-Protocol: binary\r\n\r\n'
        # Send the HTTP GET request and get the response back
        self._socket.sendall(reqdata.encode('utf8'))
        self.response = data = self._socket.recv(4096)
        # Loop through & concatenate all of the data in the response body
        end_loc = self.response.find(b'\r\n\r\n')
        while data and end_loc < 0:
            data = self._socket.recv(4096)
            self.response += data
            end_loc = self.response.find(b'\r\n\r\n')

        if len(self.response) > end_loc + 4:
            # In case some frames (e.g. the first RFP negotiation) have
            # arrived, cache it for next reading.
            self.cached_stream = self.response[end_loc + 4:]
            # ensure response ends with '\r\n\r\n'.
            self.response = self.response[:end_loc + 4]



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from urllib import parse as urllib

from oslo_serialization import jsonutils as json

from tempest.lib.common import rest_client


class RolesClient(rest_client.RestClient):
    api_version = "v2.0"

    def create_role(self, **kwargs):
        """Create a role.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/identity/v2-ext/index.html#create-a-role
        """
        post_body = json.dumps({'role': kwargs})
        resp, body = self.post('OS-KSADM/roles', post_body)
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def show_role(self, role_id_or_name):
        """Get a role by its id or name.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/identity/v2-ext/index.html#show-a-role
        OR
        https://docs.openstack.org/api-ref/identity/v2-ext/index.html#show-role-information-by-name
        """
        resp, body = self.get('OS-KSADM/roles/%s' % role_id_or_name)
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def list_roles(self, **params):
        """Returns roles.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/identity/v2-ext/index.html#list-all-roles
        """
        url = 'OS-KSADM/roles'
        if params:
            url += '?%s' % urllib.urlencode(params)
        resp, body = self.get(url)
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def delete_role(self, role_id):
        """Delete a role.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/identity/v2-ext/index.html#delete-a-role
        """
        resp, body = self.delete('OS-KSADM/roles/%s' % role_id)
        self.expected_success(204, resp.status)
        return rest_client.ResponseBody(resp, body)

    def create_user_role_on_project(self, tenant_id, user_id, role_id):
        """Add roles to a user on a tenant.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/identity/v2-ext/index.html#grant-roles-to-user-on-tenant
        """
        resp, body = self.put('/tenants/%s/users/%s/roles/OS-KSADM/%s' %
                              (tenant_id, user_id, role_id), "")
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def list_user_roles_on_project(self, tenant_id, user_id, **params):
        """Returns a list of roles assigned to a user for a tenant."""
        # TODO(gmann): Need to write API-ref link, Bug# 1592711
        url = '/tenants/%s/users/%s/roles' % (tenant_id, user_id)
        if params:
            url += '?%s' % urllib.urlencode(params)
        resp, body = self.get(url)
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def delete_role_from_user_on_project(self, tenant_id, user_id, role_id):
        """Removes a role assignment for a user on a tenant.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/identity/v2-ext/index.html#revoke-role-from-user-on-tenant
        """
        resp, body = self.delete('/tenants/%s/users/%s/roles/OS-KSADM/%s' %
                                 (tenant_id, user_id, role_id))
        self.expected_success(204, resp.status)
        return rest_client.ResponseBody(resp, body)



# Copyright 2013 IBM Corp.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import os

from oslo_concurrency import lockutils
from oslo_config import cfg
from oslo_config import fixture as conf_fixture

from tempest import config


class ConfigFixture(conf_fixture.Config):

    def __init__(self):
        cfg.CONF([], default_config_files=[])
        config.register_opts()
        super(ConfigFixture, self).__init__()

    def setUp(self):
        super(ConfigFixture, self).setUp()
        self.conf.set_default('build_interval', 10, group='compute')
        self.conf.set_default('build_timeout', 10, group='compute')
        self.conf.set_default('image_ref', 'fake_image_id', group='compute')
        self.conf.set_default('disable_ssl_certificate_validation', True,
                              group='identity')
        self.conf.set_default('uri', 'http://fake_uri.com/auth',
                              group='identity')
        self.conf.set_default('uri_v3', 'http://fake_uri_v3.com/auth',
                              group='identity')
        self.conf.set_default('neutron', True, group='service_available')
        lock_path = str(os.environ.get('OS_TEST_LOCK_PATH',
                                       os.environ.get('TMPDIR', '/tmp')))
        if not os.path.exists(lock_path):
            os.mkdir(lock_path)
        lockutils.set_defaults(
            lock_path=lock_path,
        )
        self.conf.set_default('auth_version', 'v2', group='identity')
        for config_option in ['username', 'password', 'project_name']:
            # Identity group items
            self.conf.set_default('admin_' + config_option,
                                  'fake_' + config_option,
                                  group='auth')


class FakePrivate(config.TempestConfigPrivate):
    def __init__(self, parse_conf=True, config_path=None):
        self._set_attrs()
        self.lock_path = cfg.CONF.oslo_concurrency.lock_path


fake_service1_group = cfg.OptGroup(name='fake-service1', title='Fake service1')

FakeService1Group = [
    cfg.StrOpt('catalog_type', default='fake-service1'),
    cfg.StrOpt('endpoint_type', default='faketype'),
    cfg.StrOpt('region', default='fake_region'),
    cfg.IntOpt('build_timeout', default=99),
    cfg.IntOpt('build_interval', default=9)]

fake_service2_group = cfg.OptGroup(name='fake-service2', title='Fake service2')

FakeService2Group = [
    cfg.StrOpt('catalog_type', default='fake-service2'),
    cfg.StrOpt('endpoint_type', default='faketype')]


class ServiceClientsConfigFixture(conf_fixture.Config):

    def __init__(self):
        cfg.CONF([], default_config_files=[])
        config._opts.append((fake_service1_group, FakeService1Group))
        config._opts.append((fake_service2_group, FakeService2Group))
        config.register_opts()
        super(ServiceClientsConfigFixture, self).__init__()

    def setUp(self):
        super(ServiceClientsConfigFixture, self).setUp()
        # Debug default values
        self.conf.set_default('trace_requests', 'fake_module', 'debug')
        # Identity default values
        self.conf.set_default('disable_ssl_certificate_validation', True,
                              group='identity')
        self.conf.set_default('ca_certificates_file', '/fake/certificates',
                              group='identity')
        self.conf.set_default('region', 'fake_region', 'identity')
        # Compute default values
        self.conf.set_default('build_interval', 88, group='compute')
        self.conf.set_default('build_timeout', 8, group='compute')


class ServiceClientsFakePrivate(config.TempestConfigPrivate):
    def __init__(self, parse_conf=True, config_path=None):
        self._set_attrs()
        self.fake_service1 = cfg.CONF['fake-service1']
        self.fake_service2 = cfg.CONF['fake-service2']
        self.lock_path = cfg.CONF.oslo_concurrency.lock_path



# Copyright 2018 ZTE Corporation.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

show_limits = {
    'status_code': [200],
    'response_body': {
        'type': 'object',
        'properties': {
            'limits': {
                'type': 'object',
                'properties': {
                    'rate': {'type': 'array'},
                    'absolute': {
                        'type': 'object',
                        'properties': {
                            'totalSnapshotsUsed': {'type': 'integer'},
                            'maxTotalBackups': {'type': 'integer'},
                            'maxTotalVolumeGigabytes': {'type': 'integer'},
                            'maxTotalSnapshots': {'type': 'integer'},
                            'maxTotalBackupGigabytes': {'type': 'integer'},
                            'totalBackupGigabytesUsed': {'type': 'integer'},
                            'maxTotalVolumes': {'type': 'integer'},
                            'totalVolumesUsed': {'type': 'integer'},
                            'totalBackupsUsed': {'type': 'integer'},
                            'totalGigabytesUsed': {'type': 'integer'},
                        },
                        'additionalProperties': False,
                        'required': ['totalSnapshotsUsed', 'maxTotalBackups',
                                     'maxTotalVolumeGigabytes',
                                     'maxTotalSnapshots',
                                     'maxTotalBackupGigabytes',
                                     'totalBackupGigabytesUsed',
                                     'maxTotalVolumes', 'totalVolumesUsed',
                                     'totalBackupsUsed', 'totalGigabytesUsed']
                    }
                },
                'additionalProperties': False,
                'required': ['rate', 'absolute'],
            }
        },
        'additionalProperties': False,
        'required': ['limits']
    }
}



# Copyright 2012 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.api.compute.floating_ips import base
from tempest import config
from tempest.lib import decorators

CONF = config.CONF


class FloatingIPDetailsTestJSON(base.BaseFloatingIPsTest):
    """Test floating ip details with compute microversion less than 2.36"""

    max_microversion = '2.35'

    @classmethod
    def resource_setup(cls):
        super(FloatingIPDetailsTestJSON, cls).resource_setup()
        cls.floating_ip = []
        for _ in range(3):
            body = cls.client.create_floating_ip(
                pool=CONF.network.floating_network_name)['floating_ip']
            cls.addClassResourceCleanup(cls.client.delete_floating_ip,
                                        body['id'])
            cls.floating_ip.append(body)

    @decorators.idempotent_id('16db31c3-fb85-40c9-bbe2-8cf7b67ff99f')
    def test_list_floating_ips(self):
        """Test listing floating ips"""
        body = self.client.list_floating_ips()['floating_ips']
        floating_ips = body
        self.assertNotEmpty(floating_ips,
                            "Expected floating IPs. Got zero.")
        for i in range(3):
            self.assertIn(self.floating_ip[i], floating_ips)

    @decorators.idempotent_id('eef497e0-8ff7-43c8-85ef-558440574f84')
    def test_get_floating_ip_details(self):
        """Test getting floating ip details"""
        # Creating a floating IP for which details are to be checked
        body = self.client.create_floating_ip(
            pool=CONF.network.floating_network_name)['floating_ip']
        floating_ip_id = body['id']
        self.addCleanup(self.client.delete_floating_ip,
                        floating_ip_id)
        floating_ip_instance_id = body['instance_id']
        floating_ip_ip = body['ip']
        floating_ip_fixed_ip = body['fixed_ip']
        body = self.client.show_floating_ip(floating_ip_id)['floating_ip']
        # Comparing the details of floating IP
        self.assertEqual(floating_ip_instance_id,
                         body['instance_id'])
        self.assertEqual(floating_ip_ip, body['ip'])
        self.assertEqual(floating_ip_fixed_ip,
                         body['fixed_ip'])
        self.assertEqual(floating_ip_id, body['id'])



# Copyright 2015 NEC Corporation.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.lib.services.compute import tenant_networks_client
from tempest.tests.lib import fake_auth_provider
from tempest.tests.lib.services import base


class TestTenantNetworksClient(base.BaseServiceTest):

    FAKE_NETWORK = {
        "cidr": "None",
        "id": "c2329eb4-cc8e-4439-ac4c-932369309e36",
        "label": '\u30d7'
        }

    FAKE_NETWORKS = [FAKE_NETWORK]

    NETWORK_ID = FAKE_NETWORK['id']

    def setUp(self):
        super(TestTenantNetworksClient, self).setUp()
        fake_auth = fake_auth_provider.FakeAuthProvider()
        self.client = tenant_networks_client.TenantNetworksClient(
            fake_auth, 'compute', 'regionOne')

    def _test_list_tenant_networks(self, bytes_body=False):
        self.check_service_client_function(
            self.client.list_tenant_networks,
            'tempest.lib.common.rest_client.RestClient.get',
            {"networks": self.FAKE_NETWORKS},
            bytes_body)

    def test_list_tenant_networks_with_str_body(self):
        self._test_list_tenant_networks()

    def test_list_tenant_networks_with_bytes_body(self):
        self._test_list_tenant_networks(bytes_body=True)

    def _test_show_tenant_network(self, bytes_body=False):
        self.check_service_client_function(
            self.client.show_tenant_network,
            'tempest.lib.common.rest_client.RestClient.get',
            {"network": self.FAKE_NETWORK},
            bytes_body,
            network_id=self.NETWORK_ID)

    def test_show_tenant_network_with_str_body(self):
        self._test_show_tenant_network()

    def test_show_tenant_network_with_bytes_body(self):
        self._test_show_tenant_network(bytes_body=True)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import abc

from oslo_log import log as logging
import testtools

from tempest.api.volume import base
from tempest.common import waiters
from tempest import config
from tempest.lib import decorators

CONF = config.CONF

LOG = logging.getLogger(__name__)


class VolumeRetypeTest(base.BaseVolumeAdminTest):

    def _wait_for_internal_volume_cleanup(self, vol):
        # When retyping a volume, Cinder creates an internal volume in the
        # target backend. The volume in the source backend is deleted after
        # the migration, so we need to wait for Cinder delete this volume
        # before deleting the types we've created.

        # This list should return 2 volumes until the copy and cleanup
        # process is finished.
        fetched_list = self.admin_volume_client.list_volumes(
            params={'all_tenants': True,
                    'name': vol['name']})['volumes']

        for fetched_vol in fetched_list:
            if fetched_vol['id'] != vol['id']:
                # This is the Cinder internal volume
                LOG.debug('Waiting for internal volume %s deletion',
                          fetched_vol['id'])
                self.admin_volume_client.wait_for_resource_deletion(
                    fetched_vol['id'])
                break

    @abc.abstractmethod
    def _verify_migration(self, source_vol, dest_vol):
        pass

    def _create_volume_from_snapshot(self):
        # Create a volume in the first backend
        src_vol = self.create_volume(volume_type=self.src_vol_type['name'])

        # Create a volume snapshot
        snapshot = self.create_snapshot(src_vol['id'])

        # Create a volume from the snapshot
        src_vol = self.create_volume(volume_type=self.src_vol_type['name'],
                                     snapshot_id=snapshot['id'])

        # Delete the snapshot
        self.snapshots_client.delete_snapshot(snapshot['id'])
        self.snapshots_client.wait_for_resource_deletion(snapshot['id'])

        return src_vol

    def _retype_volume(self, volume, migration_policy):

        volume_source = self.admin_volume_client.show_volume(
            volume['id'])['volume']

        self.volumes_client.retype_volume(
            volume['id'],
            new_type=self.dst_vol_type['name'],
            migration_policy=migration_policy)
        self.addCleanup(self._wait_for_internal_volume_cleanup, volume)
        waiters.wait_for_volume_retype(self.volumes_client, volume['id'],
                                       self.dst_vol_type['name'])

        volume_dest = self.admin_volume_client.show_volume(
            volume['id'])['volume']

        self._verify_migration(volume_source, volume_dest)


class VolumeRetypeWithMigrationTest(VolumeRetypeTest):
    """Test volume retype with migration"""

    @classmethod
    def skip_checks(cls):
        super(VolumeRetypeTest, cls).skip_checks()

        if not CONF.volume_feature_enabled.multi_backend:
            raise cls.skipException("Cinder multi-backend feature disabled")

        if len(set(CONF.volume.backend_names)) < 2:
            raise cls.skipException("Requires at least two different "
                                    "backend names")

    @classmethod
    def resource_setup(cls):
        super(VolumeRetypeWithMigrationTest, cls).resource_setup()
        # read backend name from a list.
        backend_src = CONF.volume.backend_names[0]
        backend_dst = CONF.volume.backend_names[1]

        extra_specs_src = {"volume_backend_name": backend_src}
        extra_specs_dst = {"volume_backend_name": backend_dst}

        cls.src_vol_type = cls.create_volume_type(extra_specs=extra_specs_src)
        cls.dst_vol_type = cls.create_volume_type(extra_specs=extra_specs_dst)

    def _verify_migration(self, volume_source, volume_dest):

        keys_with_no_change = ('id', 'size', 'description', 'name',
                               'user_id', 'os-vol-tenant-attr:tenant_id')
        keys_with_change = ('volume_type', 'os-vol-host-attr:host')

        # Check the volume information after the migration.
        self.assertEqual('success',
                         volume_dest['os-vol-mig-status-attr:migstat'])
        self.assertEqual('success', volume_dest['migration_status'])

        for key in keys_with_no_change:
            self.assertEqual(volume_source[key], volume_dest[key])

        for key in keys_with_change:
            self.assertNotEqual(volume_source[key], volume_dest[key])

        self.assertEqual(volume_dest['volume_type'], self.dst_vol_type['name'])

    @decorators.idempotent_id('a1a41f3f-9dad-493e-9f09-3ff197d477cd')
    def test_available_volume_retype_with_migration(self):
        """Test volume retype with migration

        1. Create volume1 with volume_type1
        2. Retype volume1 to volume_type2 with migration_policy='on-demand'
        3. Check volume1's volume_type is changed to volume_type2, and
           'os-vol-host-attr:host' in the volume info is changed.
        """
        src_vol = self.create_volume(volume_type=self.src_vol_type['name'])
        self._retype_volume(src_vol, migration_policy='on-demand')

    @decorators.idempotent_id('d0d9554f-e7a5-4104-8973-f35b27ccb60d')
    @testtools.skipUnless(CONF.volume_feature_enabled.snapshot,
                          "Cinder volume snapshots are disabled.")
    def test_volume_from_snapshot_retype_with_migration(self):
        """Test volume created from snapshot retype with migration

        1. Create volume1 from snapshot with volume_type1
        2. Retype volume1 to volume_type2 with migration_policy='on-demand'
        3. Check volume1's volume_type is changed to volume_type2, and
           'os-vol-host-attr:host' in the volume info is changed.
        """
        src_vol = self._create_volume_from_snapshot()

        # Migrate the volume from snapshot to the second backend
        self._retype_volume(src_vol, migration_policy='on-demand')


class VolumeRetypeWithoutMigrationTest(VolumeRetypeTest):
    """Test volume retype without migration"""

    @classmethod
    def resource_setup(cls):
        super(VolumeRetypeWithoutMigrationTest, cls).resource_setup()
        cls.src_vol_type = cls.create_volume_type('volume-type-1')
        cls.dst_vol_type = cls.create_volume_type('volume-type-2')

    def _verify_migration(self, volume_source, volume_dest):

        keys_with_no_change = ('id', 'size', 'description', 'name',
                               'user_id', 'os-vol-tenant-attr:tenant_id',
                               'os-vol-host-attr:host')
        keys_with_change = ('volume_type',)

        # Check the volume information after the retype
        self.assertIsNone(volume_dest['os-vol-mig-status-attr:migstat'])
        self.assertIsNone(volume_dest['migration_status'])

        for key in keys_with_no_change:
            self.assertEqual(volume_source[key], volume_dest[key])

        for key in keys_with_change:
            self.assertNotEqual(volume_source[key], volume_dest[key])

        self.assertEqual(volume_dest['volume_type'], self.dst_vol_type['name'])

    @decorators.idempotent_id('b90412ee-465d-46e9-b249-ec84a47d5f25')
    def test_available_volume_retype(self):
        """Test volume retype without migration

        1. Create volume1 with volume_type1
        2. Retype volume1 to volume_type2 with migration_policy='never'
        3. Check volume1's volume_type is changed to volume_type2, and
           'os-vol-host-attr:host' in the volume info is not changed.
        """
        src_vol = self.create_volume(volume_type=self.src_vol_type['name'])

        # Retype the volume from snapshot
        self._retype_volume(src_vol, migration_policy='never')



# Copyright 2017 FiberHome Telecommunication Technologies CO.,LTD
# Copyright (C) 2017 Dell Inc. or its subsidiaries.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.api.volume import base
from tempest.common import waiters
from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib.common.utils import test_utils
from tempest.lib import decorators

CONF = config.CONF


class BaseGroupSnapshotsTest(base.BaseVolumeAdminTest):

    @classmethod
    def skip_checks(cls):
        super(BaseGroupSnapshotsTest, cls).skip_checks()
        if not CONF.volume_feature_enabled.snapshot:
            raise cls.skipException("Cinder volume snapshots are disabled")

    def _create_group_snapshot(self, **kwargs):
        if 'name' not in kwargs:
            kwargs['name'] = data_utils.rand_name(
                prefix=CONF.resource_name_prefix,
                name=self.__class__.__name__ + '-Group_Snapshot')

        group_snapshot = self.group_snapshots_client.create_group_snapshot(
            **kwargs)['group_snapshot']
        group_snapshot['group_id'] = kwargs['group_id']
        self.addCleanup(test_utils.call_and_ignore_notfound_exc,
                        self._delete_group_snapshot, group_snapshot)
        waiters.wait_for_volume_resource_status(
            self.group_snapshots_client, group_snapshot['id'], 'available')
        return group_snapshot

    def _delete_group_snapshot(self, group_snapshot):
        self.group_snapshots_client.delete_group_snapshot(group_snapshot['id'])
        vols = self.volumes_client.list_volumes(detail=True)['volumes']
        snapshots = self.snapshots_client.list_snapshots(
            detail=True)['snapshots']
        for vol in vols:
            for snap in snapshots:
                if (vol['group_id'] == group_snapshot['group_id'] and
                        vol['id'] == snap['volume_id']):
                    self.snapshots_client.wait_for_resource_deletion(
                        snap['id'])
        self.group_snapshots_client.wait_for_resource_deletion(
            group_snapshot['id'])


class GroupSnapshotsTest(BaseGroupSnapshotsTest):
    """Test group snapshot"""

    volume_min_microversion = '3.14'
    volume_max_microversion = 'latest'

    @decorators.idempotent_id('1298e537-f1f0-47a3-a1dd-8adec8168897')
    def test_group_snapshot_create_show_list_delete(self):
        """Test create/show/list/delete group snapshot

        1. Create volume type "volume_type1"
        2. Create group type "group_type1"
        3. Create group "group1" with "group_type1" and "volume_type1"
        4. Create volume "volume1" with "volume_type1" and "group1"
        5. Create group snapshot "group_snapshot1" with "group1"
        6. Check snapshot created from "volume1" reaches available status
        7. Check the created group snapshot "group_snapshot1" is in the list
           of all group snapshots
        8. Delete group snapshot "group_snapshot1"
        """
        # Create volume type
        volume_type = self.create_volume_type()

        # Create group type
        group_type = self.create_group_type()

        # Create group
        grp = self.create_group(group_type=group_type['id'],
                                volume_types=[volume_type['id']])

        # Create volume is instance level, can not be deleted before group.
        # Volume delete handled by delete_group method, cleanup method.
        prefix = CONF.resource_name_prefix
        params = {'name': data_utils.rand_name(prefix=prefix, name="volume"),
                  'volume_type': volume_type['id'],
                  'group_id': grp['id'],
                  'size': CONF.volume.volume_size}
        vol = self.volumes_client.create_volume(**params)['volume']
        waiters.wait_for_volume_resource_status(
            self.volumes_client, vol['id'], 'available')

        # Create group snapshot
        group_snapshot_name = data_utils.rand_name(prefix=prefix,
                                                   name='group_snapshot')
        group_snapshot = self._create_group_snapshot(
            group_id=grp['id'], name=group_snapshot_name)
        snapshots = self.snapshots_client.list_snapshots(
            detail=True)['snapshots']
        for snap in snapshots:
            if vol['id'] == snap['volume_id']:
                waiters.wait_for_volume_resource_status(
                    self.snapshots_client, snap['id'], 'available')
        self.assertEqual(group_snapshot_name, group_snapshot['name'])

        # Get a given group snapshot
        group_snapshot = self.group_snapshots_client.show_group_snapshot(
            group_snapshot['id'])['group_snapshot']
        self.assertEqual(group_snapshot_name, group_snapshot['name'])

        # Get all group snapshots with details, check some detail-specific
        # elements, and look for the created group snapshot
        group_snapshots = self.group_snapshots_client.list_group_snapshots(
            detail=True)['group_snapshots']
        for grp_snapshot in group_snapshots:
            self.assertIn('created_at', grp_snapshot)
            self.assertIn('group_id', grp_snapshot)
        self.assertIn((group_snapshot['name'], group_snapshot['id']),
                      [(m['name'], m['id']) for m in group_snapshots])

        # Delete group snapshot
        self._delete_group_snapshot(group_snapshot)
        group_snapshots = self.group_snapshots_client.list_group_snapshots()[
            'group_snapshots']
        self.assertNotIn((group_snapshot['name'], group_snapshot['id']),
                         [(m['name'], m['id']) for m in group_snapshots])

    @decorators.idempotent_id('eff52c70-efc7-45ed-b47a-4ad675d09b81')
    def test_create_group_from_group_snapshot(self):
        """Test creating group from group snapshot

        1. Create volume type "volume_type1"
        2. Create group type "group_type1"
        3. Create group "group1" with "group_type1" and "volume_type1"
        4. Create volume "volume1" with "volume_type1" and "group1"
        5. Create group snapshot "group_snapshot1" with "group1"
        6. Check snapshot created from "volume1" reaches available status
        7. Create group "group2" from "group_snapshot1"
        8. Check the volumes belonging to "group2" reach available status
        9. Check "group2" reaches available status
        """
        # Create volume type
        volume_type = self.create_volume_type()

        # Create group type
        group_type = self.create_group_type()

        # Create Group
        grp = self.create_group(group_type=group_type['id'],
                                volume_types=[volume_type['id']])

        # Create volume is instance level, can not be deleted before group.
        # Volume delete handled by delete_group method, cleanup method.
        prefix = CONF.resource_name_prefix
        params = {'name': data_utils.rand_name(prefix=prefix, name="volume"),
                  'volume_type': volume_type['id'],
                  'group_id': grp['id'],
                  'size': CONF.volume.volume_size}
        vol = self.volumes_client.create_volume(**params)['volume']
        waiters.wait_for_volume_resource_status(
            self.volumes_client, vol['id'], 'available')

        # Create group_snapshot
        group_snapshot_name = data_utils.rand_name(prefix=prefix,
                                                   name='group_snapshot')
        group_snapshot = self._create_group_snapshot(
            group_id=grp['id'], name=group_snapshot_name)
        self.assertEqual(group_snapshot_name, group_snapshot['name'])
        snapshots = self.snapshots_client.list_snapshots(
            detail=True)['snapshots']
        for snap in snapshots:
            if vol['id'] == snap['volume_id']:
                waiters.wait_for_volume_resource_status(
                    self.snapshots_client, snap['id'], 'available')

        # Create Group from Group snapshot
        grp_name2 = data_utils.rand_name(prefix=prefix, name='Group_from_snap')
        grp2 = self.groups_client.create_group_from_source(
            group_snapshot_id=group_snapshot['id'], name=grp_name2)['group']
        self.addCleanup(self.delete_group, grp2['id'])
        self.assertEqual(grp_name2, grp2['name'])
        vols = self.volumes_client.list_volumes(detail=True)['volumes']
        for vol in vols:
            if vol['group_id'] == grp2['id']:
                waiters.wait_for_volume_resource_status(
                    self.volumes_client, vol['id'], 'available')
        waiters.wait_for_volume_resource_status(
            self.groups_client, grp2['id'], 'available')

    @decorators.idempotent_id('7d7fc000-0b4c-4376-a372-544116d2e127')
    @decorators.related_bug('1739031')
    def test_delete_group_snapshots_following_updated_volumes(self):
        """Test deleting group snapshot following updated volumes

        1. Create volume type "volume_type1"
        2. Create group type "group_type1"
        3. Create group "group1" with "group_type1" and "volume_type1"
        4. Create 2 volumes "volume1" and "volume2"
           with "volume_type1" and "group1"
        5. For each created volume, removing and then adding back to "group1"
        6. Create group snapshot "group_snapshot1" with "group1"
        7. Check snapshots created from "volume1" and "volume2" reach
           available status
        8. Delete "group_snapshot1"
        9. Check snapshots created from "volume1" and "volume2" are deleted
        """
        volume_type = self.create_volume_type()

        group_type = self.create_group_type()

        # Create a volume group
        grp = self.create_group(group_type=group_type['id'],
                                volume_types=[volume_type['id']])
        prefix = CONF.resource_name_prefix
        # Note: When dealing with consistency groups all volumes must
        # reside on the same backend. Adding volumes to the same consistency
        # group from multiple backends isn't supported. In order to ensure all
        # volumes share the same backend, all volumes must share same
        # volume-type and group id.
        volume_list = []
        for _ in range(2):
            # Create volume is instance level, can't be deleted before group.
            # Volume delete handled by delete_group method, cleanup method.
            params = {'name': data_utils.rand_name(prefix=prefix,
                                                   name="volume"),
                      'volume_type': volume_type['id'],
                      'group_id': grp['id'],
                      'size': CONF.volume.volume_size}
            volume = self.volumes_client.create_volume(**params)['volume']
            waiters.wait_for_volume_resource_status(
                self.volumes_client, volume['id'], 'available')
            volume_list.append(volume['id'])

        for vol in volume_list:
            self.groups_client.update_group(grp['id'],
                                            remove_volumes=vol)
            waiters.wait_for_volume_resource_status(
                self.groups_client, grp['id'], 'available')

            self.groups_client.update_group(grp['id'],
                                            add_volumes=vol)
            waiters.wait_for_volume_resource_status(
                self.groups_client, grp['id'], 'available')

        # Verify the created volumes are associated with consistency group
        vols = self.volumes_client.list_volumes(detail=True)['volumes']
        grp_vols = [v for v in vols if v['group_id'] == grp['id']]
        self.assertEqual(2, len(grp_vols))

        # Create a snapshot group
        group_snapshot = self._create_group_snapshot(group_id=grp['id'])
        snapshots = self.snapshots_client.list_snapshots(
            detail=True)['snapshots']

        for snap in snapshots:
            if snap['volume_id'] in volume_list:
                waiters.wait_for_volume_resource_status(
                    self.snapshots_client, snap['id'], 'available')

        # Delete a snapshot group
        self._delete_group_snapshot(group_snapshot)


class GroupSnapshotsV319Test(BaseGroupSnapshotsTest):
    """Test group snapshot with volume microversion greater than 3.18"""

    volume_min_microversion = '3.19'
    volume_max_microversion = 'latest'

    @decorators.idempotent_id('3b42c9b9-c984-4444-816e-ca2e1ed30b40')
    def test_reset_group_snapshot_status(self):
        """Test resetting group snapshot status to creating/available/error"""
        # Create volume type
        volume_type = self.create_volume_type()

        # Create group type
        group_type = self.create_group_type()

        # Create group
        group = self.create_group(group_type=group_type['id'],
                                  volume_types=[volume_type['id']])

        prefix = CONF.resource_name_prefix
        # Create volume is instance level, can not be deleted before group.
        # Volume delete handled by delete_group method, cleanup method.
        params = {'name': data_utils.rand_name(prefix=prefix, name="volume"),
                  'volume_type': volume_type['id'],
                  'group_id': group['id'],
                  'size': CONF.volume.volume_size}
        volume = self.volumes_client.create_volume(**params)['volume']
        waiters.wait_for_volume_resource_status(
            self.volumes_client, volume['id'], 'available')

        # Create group snapshot
        group_snapshot = self._create_group_snapshot(group_id=group['id'])
        snapshots = self.snapshots_client.list_snapshots(
            detail=True)['snapshots']
        for snap in snapshots:
            if volume['id'] == snap['volume_id']:
                waiters.wait_for_volume_resource_status(
                    self.snapshots_client, snap['id'], 'available')

        # Reset group snapshot status
        self.addCleanup(waiters.wait_for_volume_resource_status,
                        self.group_snapshots_client,
                        group_snapshot['id'], 'available')
        self.addCleanup(
            self.admin_group_snapshots_client.reset_group_snapshot_status,
            group_snapshot['id'], 'available')
        for status in ['creating', 'available', 'error']:
            self.admin_group_snapshots_client.reset_group_snapshot_status(
                group_snapshot['id'], status)
            waiters.wait_for_volume_resource_status(
                self.group_snapshots_client, group_snapshot['id'], status)



# Copyright 2012 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.api.compute.keypairs import base
from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators

CONF = config.CONF


class KeyPairsV2TestJSON(base.BaseKeypairTest):
    """Test keypairs API with compute microversion less than 2.2"""

    max_microversion = '2.1'

    @decorators.idempotent_id('1d1dbedb-d7a0-432a-9d09-83f543c3c19b')
    def test_keypairs_create_list_delete(self):
        """Test create/list/delete keypairs

        Keypairs created should be available in the response list
        """
        # Create 3 keypairs
        key_list = list()
        for _ in range(3):
            keypair = self.create_keypair()
            # Need to pop these keys so that our compare doesn't fail later,
            # as the keypair dicts from list API doesn't have them.
            keypair.pop('private_key')
            keypair.pop('user_id')
            key_list.append(keypair)
        # Fetch all keypairs and verify the list
        # has all created keypairs
        fetched_list = self.keypairs_client.list_keypairs()['keypairs']
        new_list = list()
        for keypair in fetched_list:
            new_list.append(keypair['keypair'])
        fetched_list = new_list
        # Now check if all the created keypairs are in the fetched list
        missing_kps = [kp for kp in key_list if kp not in fetched_list]
        self.assertFalse(missing_kps,
                         "Failed to find keypairs %s in fetched list"
                         % ', '.join(m_key['name'] for m_key in missing_kps))

    @decorators.idempotent_id('6c1d3123-4519-4742-9194-622cb1714b7d')
    def test_keypair_create_delete(self):
        """Test create/delete keypair"""
        k_name = data_utils.rand_name(
            prefix=CONF.resource_name_prefix, name='keypair')
        keypair = self.create_keypair(k_name)
        key_name = keypair['name']
        self.assertEqual(key_name, k_name,
                         "The created keypair name is not equal "
                         "to the requested name")

    @decorators.idempotent_id('a4233d5d-52d8-47cc-9a25-e1864527e3df')
    def test_get_keypair_detail(self):
        """Test getting keypair detail by keypair name"""
        k_name = data_utils.rand_name(
            prefix=CONF.resource_name_prefix, name='keypair')
        self.create_keypair(k_name)
        keypair_detail = self.keypairs_client.show_keypair(k_name)['keypair']
        self.assertEqual(keypair_detail['name'], k_name,
                         "The created keypair name is not equal "
                         "to requested name")

    @decorators.idempotent_id('39c90c6a-304a-49dd-95ec-2366129def05')
    def test_keypair_create_with_pub_key(self):
        """Test creating keypair with a given public key"""
        k_name = data_utils.rand_name(
            prefix=CONF.resource_name_prefix, name='keypair')
        pub_key = ("ssh-rsa AAAAB3NzaC1yc2EAAAADAQABAAABAQCs"
                   "Ne3/1ILNCqFyfYWDeTKLD6jEXC2OQHLmietMWW+/vd"
                   "aZq7KZEwO0jhglaFjU1mpqq4Gz5RX156sCTNM9vRbw"
                   "KAxfsdF9laBYVsex3m3Wmui3uYrKyumsoJn2g9GNnG1P"
                   "I1mrVjZ61i0GY3khna+wzlTpCCmy5HNlrmbj3XLqBUpip"
                   "TOXmsnr4sChzC53KCd8LXuwc1i/CZPvF+3XipvAgFSE53pCt"
                   "LOeB1kYMOBaiUPLQTWXR3JpckqFIQwhIH0zoHlJvZE8hh90"
                   "XcPojYN56tI0OlrGqojbediJYD0rUsJu4weZpbn8vilb3JuDY+jws"
                   "snSA8wzBx3A/8y9Pp1B nova@ubuntu")
        keypair = self.create_keypair(k_name, pub_key)
        self.assertNotIn('private_key', keypair,
                         "Field private_key is not empty!")
        key_name = keypair['name']
        self.assertEqual(key_name, k_name,
                         "The created keypair name is not equal "
                         "to the requested name!")



# Copyright 2017 AT&T Corporation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.lib.services.identity.v3 import endpoint_groups_client
from tempest.tests.lib import fake_auth_provider
from tempest.tests.lib.services import base


class TestEndPointGroupsClient(base.BaseServiceTest):
    FAKE_CREATE_ENDPOINT_GROUP = {
        "endpoint_group": {
            "id": 1,
            "name": "FAKE_ENDPOINT_GROUP",
            "description": "FAKE SERVICE ENDPOINT GROUP",
            "filters": {
                "service_id": 1
            }
        }
    }

    FAKE_ENDPOINT_GROUP_INFO = {
        "endpoint_group": {
            "id": 1,
            "name": "FAKE_ENDPOINT_GROUP",
            "description": "FAKE SERVICE ENDPOINT GROUP",
            "links": {
                "self": "http://example.com/identity/v3/OS-EP-FILTER/" +
                        "endpoint_groups/1"
            },
            "filters": {
                "service_id": 1
            }
        }
    }

    FAKE_LIST_ENDPOINT_GROUPS = {
        "endpoint_groups": [
            {
                "id": 1,
                "name": "SERVICE_GROUP1",
                "description": "FAKE SERVICE ENDPOINT GROUP",
                "links": {
                    "self": "http://example.com/identity/v3/OS-EP-FILTER/" +
                            "endpoint_groups/1"
                },
                "filters": {
                    "service_id": 1
                }
            },
            {
                "id": 2,
                "name": "SERVICE_GROUP2",
                "description": "FAKE SERVICE ENDPOINT GROUP",
                "links": {
                    "self": "http://example.com/identity/v3/OS-EP-FILTER/" +
                            "endpoint_groups/2"
                },
                "filters": {
                    "service_id": 2
                }
            }
        ]
    }

    def setUp(self):
        super(TestEndPointGroupsClient, self).setUp()
        fake_auth = fake_auth_provider.FakeAuthProvider()
        self.client = endpoint_groups_client.EndPointGroupsClient(
            fake_auth, 'identity', 'regionOne')

    def _test_create_endpoint_group(self, bytes_body=False):
        self.check_service_client_function(
            self.client.create_endpoint_group,
            'tempest.lib.common.rest_client.RestClient.post',
            self.FAKE_CREATE_ENDPOINT_GROUP,
            bytes_body,
            status=201,
            name="FAKE_ENDPOINT_GROUP",
            filters={'service_id': "1"})

    def _test_show_endpoint_group(self, bytes_body=False):
        self.check_service_client_function(
            self.client.show_endpoint_group,
            'tempest.lib.common.rest_client.RestClient.get',
            self.FAKE_ENDPOINT_GROUP_INFO,
            bytes_body,
            endpoint_group_id="1")

    def _test_check_endpoint_group(self, bytes_body=False):
        self.check_service_client_function(
            self.client.check_endpoint_group,
            'tempest.lib.common.rest_client.RestClient.head',
            {},
            bytes_body,
            status=200,
            endpoint_group_id="1")

    def _test_update_endpoint_group(self, bytes_body=False):
        self.check_service_client_function(
            self.client.update_endpoint_group,
            'tempest.lib.common.rest_client.RestClient.patch',
            self.FAKE_ENDPOINT_GROUP_INFO,
            bytes_body,
            endpoint_group_id="1",
            name="NewName")

    def _test_list_endpoint_groups(self, bytes_body=False):
        self.check_service_client_function(
            self.client.list_endpoint_groups,
            'tempest.lib.common.rest_client.RestClient.get',
            self.FAKE_LIST_ENDPOINT_GROUPS,
            bytes_body)

    def test_create_endpoint_group_with_str_body(self):
        self._test_create_endpoint_group()

    def test_create_endpoint_group_with_bytes_body(self):
        self._test_create_endpoint_group(bytes_body=True)

    def test_show_endpoint_group_with_str_body(self):
        self._test_show_endpoint_group()

    def test_show_endpoint_group_with_bytes_body(self):
        self._test_show_endpoint_group(bytes_body=True)

    def test_check_endpoint_group_with_str_body(self):
        self._test_check_endpoint_group()

    def test_check_endpoint_group_with_bytes_body(self):
        self._test_check_endpoint_group(bytes_body=True)

    def test_list_endpoint_groups_with_str_body(self):
        self._test_list_endpoint_groups()

    def test_list_endpoint_groups_with_bytes_body(self):
        self._test_list_endpoint_groups(bytes_body=True)

    def test_update_endpoint_group_with_str_body(self):
        self._test_update_endpoint_group()

    def test_update_endpoint_group_with_bytes_body(self):
        self._test_update_endpoint_group(bytes_body=True)

    def test_delete_endpoint_group(self):
        self.check_service_client_function(
            self.client.delete_endpoint_group,
            'tempest.lib.common.rest_client.RestClient.delete',
            {},
            endpoint_group_id="1",
            status=204)



# Copyright 2014 NEC Corporation.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import copy

from tempest.lib.api_schema.response.compute.v2_1 import parameter_types

image_links = copy.deepcopy(parameter_types.links)
image_links['items']['properties'].update({'type': {'type': 'string'}})

image_status_enums = ['ACTIVE', 'SAVING', 'DELETED', 'ERROR', 'UNKNOWN']

common_image_schema = {
    'type': 'object',
    'properties': {
        'id': {'type': 'string'},
        'status': {'enum': image_status_enums},
        'updated': parameter_types.date_time,
        'links': image_links,
        'name': {'type': ['string', 'null']},
        'created': parameter_types.date_time,
        'minDisk': {'type': 'integer'},
        'minRam': {'type': 'integer'},
        'progress': {'type': 'integer'},
        'metadata': {'type': 'object'},
        'server': {
            'type': 'object',
            'properties': {
                'id': {'type': 'string'},
                'links': parameter_types.links
            },
            'additionalProperties': False,
            'required': ['id', 'links']
        },
        'OS-EXT-IMG-SIZE:size': {'type': ['integer', 'null']},
        'OS-DCF:diskConfig': {'type': 'string'}
    },
    'additionalProperties': False,
    # 'server' attributes only comes in response body if image is
    # associated with any server. 'OS-EXT-IMG-SIZE:size' & 'OS-DCF:diskConfig'
    # are API extension,  So those are not defined as 'required'.
    'required': ['id', 'status', 'updated', 'links', 'name',
                 'created', 'minDisk', 'minRam', 'progress',
                 'metadata']
}

get_image = {
    'status_code': [200],
    'response_body': {
        'type': 'object',
        'properties': {
            'image': common_image_schema
        },
        'additionalProperties': False,
        'required': ['image']
    }
}

list_images = {
    'status_code': [200],
    'response_body': {
        'type': 'object',
        'properties': {
            'images': {
                'type': 'array',
                'items': {
                    'type': 'object',
                    'properties': {
                        'id': {'type': 'string'},
                        'links': image_links,
                        'name': {'type': ['string', 'null']}
                    },
                    'additionalProperties': False,
                    'required': ['id', 'links', 'name']
                }
            },
            'images_links': parameter_types.links
        },
        'additionalProperties': False,
        # NOTE(gmann): images_links attribute is not necessary to be
        # present always So it is not 'required'.
        'required': ['images']
    }
}

create_image = {
    'status_code': [202],
    'response_header': {
        'type': 'object',
        'properties': parameter_types.response_header
    }
}
create_image['response_header']['properties'].update(
    {'location': {
        'type': 'string',
        'format': 'uri'}
     }
)
create_image['response_header']['required'] = ['location']

delete = {
    'status_code': [204]
}

image_metadata = {
    'status_code': [200],
    'response_body': {
        'type': 'object',
        'properties': {
            'metadata': {'type': 'object'}
        },
        'additionalProperties': False,
        'required': ['metadata']
    }
}

image_meta_item = {
    'status_code': [200],
    'response_body': {
        'type': 'object',
        'properties': {
            'meta': {'type': 'object'}
        },
        'additionalProperties': False,
        'required': ['meta']
    }
}

list_images_details = {
    'status_code': [200],
    'response_body': {
        'type': 'object',
        'properties': {
            'images': {
                'type': 'array',
                'items': common_image_schema
            },
            'images_links': parameter_types.links
        },
        'additionalProperties': False,
        # NOTE(gmann): images_links attribute is not necessary to be
        # present always So it is not 'required'.
        'required': ['images']
    }
}



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import copy

from tempest.lib.api_schema.response.compute.v2_59 import migrations

###########################################################################
#
# 2.80:
#
# The user_id and project_id value is now returned in the response body in
# addition to the migration id for the following API responses:
#
# - GET /os-migrations
#
###########################################################################

user_id = {'type': 'string'}
project_id = {'type': 'string'}

list_migrations = copy.deepcopy(migrations.list_migrations)

list_migrations['response_body']['properties']['migrations']['items'][
    'properties'].update({
        'user_id': user_id,
        'project_id': project_id
    })

list_migrations['response_body']['properties']['migrations']['items'][
    'required'].extend(['user_id', 'project_id'])



# Copyright 2014 Hewlett-Packard Development Company, L.P.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.


class FakeAuthProvider(object):

    def __init__(self, creds_dict=None, fake_base_url=None):
        creds_dict = creds_dict or {}
        self.credentials = FakeCredentials(creds_dict)
        self.fake_base_url = fake_base_url

    def auth_request(self, method, url, headers=None, body=None, filters=None):
        return url, headers, body

    def base_url(self, filters, auth_data=None):
        return self.fake_base_url or "https://example.com"

    def get_token(self):
        return "faketoken"


class FakeCredentials(object):

    def __init__(self, creds_dict):
        for key in creds_dict:
            setattr(self, key, creds_dict[key])



# Copyright 2012 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from urllib import parse as urllib

from defusedxml import ElementTree as etree
from oslo_serialization import jsonutils as json

from tempest.lib.common import rest_client


class AccountClient(rest_client.RestClient):

    def create_update_or_delete_account_metadata(
            self,
            create_update_metadata=None,
            delete_metadata=None,
            create_update_metadata_prefix='X-Account-Meta-',
            delete_metadata_prefix='X-Remove-Account-Meta-'):
        """Creates, Updates or deletes an account metadata entry.

        Account Metadata can be created, updated or deleted based on
        metadata header or value. For detailed info, please refer to the
        official API reference:
        https://docs.openstack.org/api-ref/object-store/#create-update-or-delete-account-metadata
        """
        headers = {}
        if create_update_metadata:
            for key in create_update_metadata:
                metadata_header_name = create_update_metadata_prefix + \
                    key.replace('_', '-')
                headers[metadata_header_name] = create_update_metadata[key]
        if delete_metadata:
            for key in delete_metadata:
                headers[delete_metadata_prefix + key.replace(
                    '_', '-')] = delete_metadata[key]

        resp, body = self.post('', headers=headers, body=None)
        self.expected_success([200, 204], resp.status)
        return resp, body

    def list_account_metadata(self):
        """List all account metadata."""
        resp, body = self.head('')
        self.expected_success(204, resp.status)
        return resp, body

    def list_account_containers(self, params=None):
        """List all containers for the account.

        Given valid X-Auth-Token, returns a list of all containers for the
        account.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/object-store/#show-account-details-and-list-containers
        """
        url = '?%s' % urllib.urlencode(params) if params else ''

        resp, body = self.get(url, headers={})
        if params and params.get('format') == 'json':
            body = json.loads(body)
        elif params and params.get('format') == 'xml':
            body = etree.fromstring(body)
        else:
            body = body.strip().splitlines()
        self.expected_success([200, 204], resp.status)
        return resp, body



# Copyright 2018 ZTE Corporation.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

property_info = {
    'type': 'object',
    'properties': {
        'type': {'type': 'string'},
        'description': {'type': 'string'},
        'title': {'type': 'string'}
    },
    'additionalProperties': False,
    'required': ['type', 'description', 'title']
}

show_backend_capabilities = {
    'status_code': [200],
    'response_body': {
        'type': 'object',
        'properties': {
            'pool_name': {'type': ['string', 'null']},
            'description': {'type': ['string', 'null']},
            'volume_backend_name': {'type': 'string'},
            'namespace': {'type': 'string',
                          'pattern': '^OS::Storage::Capabilities::.+$'},
            'visibility': {'type': ['string', 'null']},
            'driver_version': {'type': 'string'},
            'vendor_name': {'type': 'string'},
            'properties': {
                'type': 'object',
                'properties': {
                    '^.+$': property_info
                },
            },
            'storage_protocol': {'type': 'string'},
            'replication_targets': {'type': 'array'},
            'display_name': {'type': ['string', 'null']}
        },
        'additionalProperties': False,
        'required': ['pool_name', 'volume_backend_name', 'namespace',
                     'visibility', 'driver_version', 'vendor_name',
                     'properties', 'storage_protocol', 'replication_targets',
                     'display_name', 'description']
    }
}



# Copyright 2012 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.api.compute import base
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc


class ServerAddressesNegativeTestJSON(base.BaseV2ComputeTest):
    """Negative tests of listing server addresses"""
    create_default_network = True

    @classmethod
    def setup_clients(cls):
        super(ServerAddressesNegativeTestJSON, cls).setup_clients()
        cls.client = cls.servers_client

    @classmethod
    def resource_setup(cls):
        super(ServerAddressesNegativeTestJSON, cls).resource_setup()
        cls.server = cls.create_test_server(wait_until='ACTIVE')

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('02c3f645-2d2e-4417-8525-68c0407d001b')
    def test_list_server_addresses_invalid_server_id(self):
        """List addresses request should fail if server id not in system"""
        self.assertRaises(lib_exc.NotFound, self.client.list_addresses,
                          '999')

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('a2ab5144-78c0-4942-a0ed-cc8edccfd9ba')
    def test_list_server_addresses_by_network_neg(self):
        """List addresses by network should fail if network name not valid"""
        self.assertRaises(lib_exc.NotFound,
                          self.client.list_addresses_by_network,
                          self.server['id'], 'invalid')



# Copyright 2022 Red Hat, Inc.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.lib.services.image.v2 import image_cache_client
from tempest.tests.lib import fake_auth_provider
from tempest.tests.lib.services import base


class TestImageCacheClient(base.BaseServiceTest):
    def setUp(self):
        super(TestImageCacheClient, self).setUp()
        fake_auth = fake_auth_provider.FakeAuthProvider()
        self.client = image_cache_client.ImageCacheClient(
            fake_auth, 'image', 'regionOne')

    def test_list_cache(self):
        fake_result = {
            "cached_images": [{
                "image_id": "8f332e84-ea60-4501-8e11-5efcddb81f30",
                "hits": 3,
                "last_accessed": 1639578364.65118,
                "last_modified": 1639389612.596718,
                "size": 16300544
            }],
            "queued_images": ['1bea47ed-f6a9-463b-b423-14b9cca9ad27']}
        self.check_service_client_function(
            self.client.list_cache,
            'tempest.lib.common.rest_client.RestClient.get',
            fake_result,
            mock_args=['cache'])

    def test_cache_queue(self):
        self.check_service_client_function(
            self.client.cache_queue,
            'tempest.lib.common.rest_client.RestClient.put',
            {},
            status=202,
            image_id="e485aab9-0907-4973-921c-bb6da8a8fcf8")

    def test_cache_delete(self):
        fake_result = {}
        self.check_service_client_function(
            self.client.cache_delete,
            'tempest.lib.common.rest_client.RestClient.delete',
            fake_result, image_id="e485aab9-0907-4973-921c-bb6da8a8fcf8",
            status=204)

    def test_cache_clear_without_target(self):
        fake_result = {}
        self.check_service_client_function(
            self.client.cache_clear,
            'tempest.lib.common.rest_client.RestClient.delete',
            fake_result, status=204)



# Copyright 2014 NEC Corporation.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.lib.api_schema.response.compute.v2_1 import parameter_types

list_services = {
    'status_code': [200],
    'response_body': {
        'type': 'object',
        'properties': {
            'services': {
                'type': 'array',
                'items': {
                    'type': 'object',
                    'properties': {
                        'id': {'type': ['integer', 'string'],
                               'pattern': '^[a-zA-Z!]*@[0-9]+$'},
                        'zone': {'type': 'string'},
                        'host': {'type': 'string'},
                        'state': {'type': 'string'},
                        'binary': {'type': 'string'},
                        'status': {'type': 'string'},
                        'updated_at': parameter_types.date_time_or_null,
                        'disabled_reason': {'type': ['string', 'null']}
                    },
                    'additionalProperties': False,
                    'required': ['id', 'zone', 'host', 'state', 'binary',
                                 'status', 'updated_at', 'disabled_reason']
                }
            }
        },
        'additionalProperties': False,
        'required': ['services']
    }
}

enable_disable_service = {
    'status_code': [200],
    'response_body': {
        'type': 'object',
        'properties': {
            'service': {
                'type': 'object',
                'properties': {
                    'status': {'type': 'string'},
                    'binary': {'type': 'string'},
                    'host': {'type': 'string'}
                },
                'additionalProperties': False,
                'required': ['status', 'binary', 'host']
            }
        },
        'additionalProperties': False,
        'required': ['service']
    }
}

disable_log_reason = {
    'status_code': [200],
    'response_body': {
        'type': 'object',
        'properties': {
            'service': {
                'type': 'object',
                'properties': {
                    'disabled_reason': {'type': 'string'},
                    'binary': {'type': 'string'},
                    'host': {'type': 'string'},
                    'status': {'type': 'string'}
                },
                'additionalProperties': False,
                'required': ['disabled_reason', 'binary', 'host', 'status']
            }
        },
        'additionalProperties': False,
        'required': ['service']
    }
}



# Copyright 2017 Mirantis Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import time

from tempest.api.compute import base
from tempest.lib import decorators


class ServerDiagnosticsTest(base.BaseV2ComputeAdminTest):
    """Test server diagnostics with compute microversion less than 2.48"""

    min_microversion = None
    max_microversion = '2.47'

    @classmethod
    def setup_clients(cls):
        super(ServerDiagnosticsTest, cls).setup_clients()
        cls.client = cls.os_admin.servers_client

    @decorators.idempotent_id('31ff3486-b8a0-4f56-a6c0-aab460531db3')
    def test_get_server_diagnostics(self):
        """Test getting server diagnostics"""
        server_id = self.create_test_server(wait_until='ACTIVE')['id']
        diagnostics = self.client.show_server_diagnostics(server_id)

        # NOTE(snikitin): Before microversion 2.48 response data from each
        # hypervisor (libvirt, xen, vmware) was different. None of the fields
        # were equal. As this test is common for libvirt, xen and vmware CI
        # jobs we can't check any field in the response because all fields are
        # different.
        self.assertNotEmpty(diagnostics)


class ServerDiagnosticsV248Test(base.BaseV2ComputeAdminTest):
    """Test server diagnostics with compute microversion greater than 2.47"""

    min_microversion = '2.48'
    max_microversion = 'latest'

    @classmethod
    def setup_clients(cls):
        super(ServerDiagnosticsV248Test, cls).setup_clients()
        cls.client = cls.os_admin.servers_client

    @decorators.idempotent_id('64d0d48c-dff1-11e6-bf01-fe55135034f3')
    def test_get_server_diagnostics(self):
        """Test getting server diagnostics"""
        server_id = self.create_test_server(wait_until='ACTIVE')['id']
        # Response status and filed types will be checked by json schema
        self.client.show_server_diagnostics(server_id)

        # NOTE(snikitin): This is a special case for Xen hypervisor. In Xen
        # case we're getting diagnostics stats from the RRDs which are updated
        # every 5 seconds. It means that diagnostics information may be
        # incomplete during first 5 seconds of VM life. In such cases methods
        # which get diagnostics stats from Xen may raise exceptions or
        # return `NaN` values. Such behavior must be handled correctly.
        # Response must contain all diagnostics fields (may be with `None`
        # values) and response status must be 200. Line above checks it by
        # json schema.
        time.sleep(10)
        diagnostics = self.client.show_server_diagnostics(server_id)

        # NOTE(snikitin): After 10 seconds diagnostics fields must contain
        # not None values. But we will check only "memory_details.maximum"
        # field because only this field meets all the following conditions:
        # 1) This field may be unset because of Xen 5 seconds timeout.
        # 2) This field is present in responses from all three supported
        #    hypervisors (libvirt, xen, vmware).
        self.assertIsNotNone(diagnostics['memory_details']['maximum'])



# Copyright (C) 2016, Red Hat, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from json import decoder as json_decoder

from oslo_log import log as logging
from oslo_serialization import jsonutils as json

from tempest.api.compute import base
from tempest.common import utils
from tempest.common.utils.linux import remote_client
from tempest.common import waiters
from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators


CONF = config.CONF

LOG = logging.getLogger(__name__)


class DeviceTaggingBase(base.BaseV2ComputeTest):

    credentials = ['primary', 'admin']

    @classmethod
    def skip_checks(cls):
        super(DeviceTaggingBase, cls).skip_checks()
        if not CONF.service_available.neutron:
            raise cls.skipException('Neutron is required')
        if not CONF.validation.run_validation:
            raise cls.skipException('Validation must be enabled')
        if (not CONF.compute_feature_enabled.config_drive and
                not CONF.compute_feature_enabled.metadata_service):
            raise cls.skipException('One of metadata or config drive must be '
                                    'enabled')

    @classmethod
    def setup_clients(cls):
        super(DeviceTaggingBase, cls).setup_clients()
        cls.networks_client = cls.os_primary.networks_client
        cls.ports_client = cls.os_primary.ports_client
        cls.subnets_client = cls.os_primary.subnets_client
        cls.interfaces_client = cls.os_primary.interfaces_client
        cls.servers_admin_client = cls.os_admin.servers_client

    @classmethod
    def setup_credentials(cls):
        cls.set_network_resources(network=True, subnet=True, router=True,
                                  dhcp=True)
        super(DeviceTaggingBase, cls).setup_credentials()

    def verify_metadata_on_config_drive(self, server, ssh_client,
                                        verify_method):
        LOG.info('Attempting to verify tagged devices in server %s via '
                 'the config drive.', server['id'])
        ssh_client.mount_config_drive()
        cmd_md = 'sudo cat /mnt/openstack/latest/meta_data.json'
        md_json = ssh_client.exec_command(cmd_md)
        verify_method(md_json)
        ssh_client.unmount_config_drive()


class TaggedBootDevicesTest(DeviceTaggingBase):
    """Test tagged boot device with compute microversion equals 2.32"""

    min_microversion = '2.32'
    # NOTE(mriedem): max_version looks odd but it's actually correct. Due to a
    # bug in the 2.32 microversion, tags on block devices only worked with the
    # 2.32 microversion specifically. And tags on networks only worked between
    # 2.32 and 2.36 inclusive; the 2.37 microversion broke tags for networks.
    max_microversion = '2.32'

    def verify_device_metadata(self, md_json):
        try:
            md_dict = json.loads(md_json)
        except (json_decoder.JSONDecodeError, TypeError):
            return False

        for d in md_dict['devices']:
            if d['type'] == 'nic':
                if d['mac'] == self.port1['mac_address']:
                    self.assertEqual(d['tags'], ['port-1'])
                if d['mac'] == self.port2['mac_address']:
                    self.assertEqual(d['tags'], ['port-2'])
                if d['mac'] == self.net_2_100_mac:
                    self.assertEqual(d['tags'], ['net-2-100'])
                if d['mac'] == self.net_2_200_mac:
                    self.assertEqual(d['tags'], ['net-2-200'])

        # A hypervisor may present multiple paths to a tagged disk, so
        # there may be duplicated tags in the metadata, use set() to
        # remove duplicated tags.
        # Some hypervisors might report devices with no tags as well.
        found_devices = [d['tags'][0] for d in md_dict['devices']
                         if d.get('tags')]
        try:
            self.assertEqual(set(found_devices), set(['port-1', 'port-2',
                                                      'net-1', 'net-2-100',
                                                      'net-2-200', 'boot',
                                                      'other']))
            return True
        except Exception:
            return False

    # NOTE(mriedem): This is really more like a scenario test and is slow so
    # it's marked as such.
    @decorators.attr(type='slow')
    @decorators.idempotent_id('a2e65a6c-66f1-4442-aaa8-498c31778d96')
    @utils.services('network', 'volume', 'image')
    def test_tagged_boot_devices(self):
        """Test tagged boot devices

        1. Create volumes
        2. Create networks
        3. Create subnets
        4. Create ports
        5. Create server, specifying tags for items in networks and
           block_device_mapping_v2,
        6. Verify tagged devices are in server via metadata service
        """
        # Create volumes
        # The create_volume methods waits for the volumes to be available and
        # the base class will clean them up on tearDown.
        boot_volume = self.create_volume(CONF.compute.image_ref)
        other_volume = self.create_volume()
        untagged_volume = self.create_volume()

        # Create networks
        net1 = self.networks_client.create_network(
            name=data_utils.rand_name(
                prefix=CONF.resource_name_prefix,
                name='device-tagging-net1'))['network']
        self.addCleanup(self.networks_client.delete_network, net1['id'])

        net2 = self.networks_client.create_network(
            name=data_utils.rand_name(
                prefix=CONF.resource_name_prefix,
                name='device-tagging-net2'))['network']
        self.addCleanup(self.networks_client.delete_network, net2['id'])

        # Create subnets
        subnet1 = self.subnets_client.create_subnet(
            network_id=net1['id'],
            cidr='10.1.1.0/24',
            ip_version=4)['subnet']
        self.addCleanup(self.subnets_client.delete_subnet, subnet1['id'])

        subnet2 = self.subnets_client.create_subnet(
            network_id=net2['id'],
            cidr='10.2.2.0/24',
            ip_version=4)['subnet']
        self.addCleanup(self.subnets_client.delete_subnet, subnet2['id'])

        # Create ports
        self.port1 = self.ports_client.create_port(
            network_id=net1['id'],
            name=data_utils.rand_name(
                prefix=CONF.resource_name_prefix,
                name=self.__class__.__name__),
            fixed_ips=[{'subnet_id': subnet1['id']}])['port']
        self.addCleanup(self.ports_client.delete_port, self.port1['id'])

        self.port2 = self.ports_client.create_port(
            network_id=net1['id'],
            name=data_utils.rand_name(
                prefix=CONF.resource_name_prefix,
                name=self.__class__.__name__),
            fixed_ips=[{'subnet_id': subnet1['id']}])['port']
        self.addCleanup(self.ports_client.delete_port, self.port2['id'])

        # Create server
        config_drive_enabled = CONF.compute_feature_enabled.config_drive
        validation_resources = self.get_test_validation_resources(
            self.os_primary)

        server = self.create_test_server(
            validatable=True,
            wait_until='SSHABLE',
            validation_resources=validation_resources,
            config_drive=config_drive_enabled,
            name=data_utils.rand_name(
                prefix=CONF.resource_name_prefix,
                name='device-tagging-server'),
            networks=[
                # Validation network for ssh
                {
                    'uuid': self.get_tenant_network()['id']
                },
                # Different tags for different ports
                {
                    'port': self.port1['id'],
                    'tag': 'port-1'
                },
                {
                    'port': self.port2['id'],
                    'tag': 'port-2'
                },
                # Two nics on same net, one tagged one not
                {
                    'uuid': net1['id'],
                    'tag': 'net-1'
                },
                {
                    'uuid': net1['id']
                },
                # Two nics on same net, different IP
                {
                    'uuid': net2['id'],
                    'fixed_ip': '10.2.2.100',
                    'tag': 'net-2-100'
                },
                {
                    'uuid': net2['id'],
                    'fixed_ip': '10.2.2.200',
                    'tag': 'net-2-200'
                }
            ],
            block_device_mapping_v2=[
                # Boot volume
                {
                    'uuid': boot_volume['id'],
                    'source_type': 'volume',
                    'destination_type': 'volume',
                    'boot_index': 0,
                    'tag': 'boot'
                },
                # Other volume
                {
                    'uuid': other_volume['id'],
                    'source_type': 'volume',
                    'destination_type': 'volume',
                    'boot_index': 1,
                    'tag': 'other'
                },
                # Untagged volume
                {
                    'uuid': untagged_volume['id'],
                    'source_type': 'volume',
                    'destination_type': 'volume',
                    'boot_index': 2
                }
            ])

        self.addCleanup(self.delete_server, server['id'])

        server = self.servers_client.show_server(server['id'])['server']
        ssh_client = remote_client.RemoteClient(
            self.get_server_ip(server, validation_resources),
            CONF.validation.image_ssh_user,
            pkey=validation_resources['keypair']['private_key'],
            server=server,
            servers_client=self.servers_client)

        # Find the MAC addresses of our fixed IPs
        self.net_2_100_mac = None
        self.net_2_200_mac = None
        ifaces = self.interfaces_client.list_interfaces(server['id'])
        for iface in ifaces['interfaceAttachments']:
            if 'fixed_ips' in iface:
                for ip in iface['fixed_ips']:
                    if ip['ip_address'] == '10.2.2.100':
                        self.net_2_100_mac = iface['mac_addr']
                    if ip['ip_address'] == '10.2.2.200':
                        self.net_2_200_mac = iface['mac_addr']
        # Make sure we have the MACs we need, there's no reason for some to be
        # missing
        self.assertTrue(self.net_2_100_mac)
        self.assertTrue(self.net_2_200_mac)

        # Verify metadata from metadata API
        if CONF.compute_feature_enabled.metadata_service:
            self.verify_metadata_from_api(server, ssh_client,
                                          self.verify_device_metadata)

        # Verify metadata on config drive
        if CONF.compute_feature_enabled.config_drive:
            self.verify_metadata_on_config_drive(server, ssh_client,
                                                 self.verify_device_metadata)


class TaggedBootDevicesTest_v242(TaggedBootDevicesTest):
    """Test tagged boot devices with compute microversion greater than 2.41"""

    min_microversion = '2.42'
    max_microversion = 'latest'


class TaggedAttachmentsTest(DeviceTaggingBase):
    """Test tagged attachments with compute microversion greater than 2.48"""

    min_microversion = '2.49'
    max_microversion = 'latest'

    @classmethod
    def skip_checks(cls):
        super(TaggedAttachmentsTest, cls).skip_checks()
        if not CONF.compute_feature_enabled.metadata_service:
            raise cls.skipException('Metadata API must be enabled')

    def verify_device_metadata(self, md_json):
        try:
            md_dict = json.loads(md_json)
        except (json_decoder.JSONDecodeError, TypeError) as e:
            LOG.warning(
                'Failed to decode json metadata: %s, %s', str(e), str(md_json))
            return False

        found_devices = [d['tags'][0] for d in md_dict['devices']
                         if d.get('tags')]
        try:
            self.assertEqual(
                sorted(found_devices),
                sorted(['nic-tag', 'volume-tag']))
            return True
        except Exception as e:
            LOG.warning(
                'Failed to parse metadata: %s, %s', str(e), str(md_json))
            return False

    def verify_empty_devices(self, md_json):
        md_dict = json.loads(md_json)
        try:
            self.assertEmpty(md_dict['devices'])
            return True
        except AssertionError:
            LOG.debug("Related bug 1775947. Devices dict is not empty: %s",
                      md_dict['devices'])
            return False

    @decorators.idempotent_id('3e41c782-2a89-4922-a9d2-9a188c4e7c7c')
    @utils.services('network', 'volume', 'image')
    def test_tagged_attachment(self):
        """Test tagged attachment

        1. Create network
        2. Create subnet
        3. Create volume
        4. Create server
        5. Attach tagged networks and volume
        6. Verify tagged devices are in server via metadata service
        7. Detach tagged networks and volume
        """
        # Create network
        net = self.networks_client.create_network(
            name=data_utils.rand_name(
                prefix=CONF.resource_name_prefix,
                name='tagged-attachments-test-net'))['network']
        self.addCleanup(self.networks_client.delete_network, net['id'])

        # Create subnet
        subnet = self.subnets_client.create_subnet(
            network_id=net['id'],
            cidr='10.10.10.0/24',
            ip_version=4)['subnet']
        self.addCleanup(self.subnets_client.delete_subnet, subnet['id'])

        # Create volume
        volume = self.create_volume()

        # Boot test server
        config_drive_enabled = CONF.compute_feature_enabled.config_drive
        validation_resources = self.get_test_validation_resources(
            self.os_primary)

        server = self.create_test_server(
            validatable=True,
            validation_resources=validation_resources,
            config_drive=config_drive_enabled,
            name=data_utils.rand_name(
                prefix=CONF.resource_name_prefix,
                name='device-tagging-server'),
            networks=[{'uuid': self.get_tenant_network()['id']}],
            wait_until='SSHABLE')
        self.addCleanup(self.delete_server, server['id'])

        # NOTE(mgoddard): Get detailed server to ensure addresses are present
        # in fixed IP case.
        server = self.servers_client.show_server(server['id'])['server']

        # Attach tagged nic and volume
        interface = self.interfaces_client.create_interface(
            server['id'], net_id=net['id'],
            tag='nic-tag')['interfaceAttachment']
        self.attach_volume(server, volume, tag='volume-tag')

        ssh_client = remote_client.RemoteClient(
            self.get_server_ip(server, validation_resources),
            CONF.validation.image_ssh_user,
            pkey=validation_resources['keypair']['private_key'],
            server=server,
            servers_client=self.servers_client)

        self.verify_metadata_from_api(server, ssh_client,
                                      self.verify_device_metadata)

        # Detach tagged nic and volume
        self.servers_client.detach_volume(server['id'], volume['id'])
        waiters.wait_for_volume_resource_status(self.volumes_client,
                                                volume['id'], 'available')
        req_id = self.interfaces_client.delete_interface(
            server['id'], interface['port_id']
        ).response['x-openstack-request-id']
        waiters.wait_for_interface_detach(self.servers_admin_client,
                                          server['id'],
                                          interface['port_id'],
                                          req_id)
        # FIXME(mriedem): The assertion that the tagged devices are removed
        # from the metadata for the server is being skipped until bug 1775947
        # is fixed.
        # self.verify_metadata_from_api(server, ssh_client,
        #                               self.verify_empty_devices)



# Copyright 2013 Huawei Technologies Co.,LTD.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import testtools

from tempest.api.compute.security_groups import base
from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc

CONF = config.CONF


class SecurityGroupsNegativeTestJSON(base.BaseSecurityGroupsTest):
    """Negative tests of security groups API

    Negative tests of security groups API with compute microversion
    less than 2.36.
    """

    @classmethod
    def setup_clients(cls):
        super(SecurityGroupsNegativeTestJSON, cls).setup_clients()
        cls.client = cls.security_groups_client

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('673eaec1-9b3e-48ed-bdf1-2786c1b9661c')
    def test_security_group_get_nonexistent_group(self):
        """Test getting non existent security group details should fail"""
        non_exist_id = self.generate_random_security_group_id()
        self.assertRaises(lib_exc.NotFound, self.client.show_security_group,
                          non_exist_id)

    @decorators.skip_because(bug="1161411",
                             condition=CONF.service_available.neutron)
    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('1759c3cb-b0fc-44b7-86ce-c99236be911d')
    def test_security_group_create_with_invalid_group_name(self):
        """Test creating security group with invalid group name should fail

        Negative test: Security group should not be created with group name
        as an empty string, or group name with white spaces, or group name
        with chars more than 255.
        """
        s_description = data_utils.rand_name(
            prefix=CONF.resource_name_prefix, name='description')
        # Create Security Group with empty string as group name
        self.assertRaises(lib_exc.BadRequest,
                          self.client.create_security_group,
                          name="", description=s_description)
        # Create Security Group with white space in group name
        self.assertRaises(lib_exc.BadRequest,
                          self.client.create_security_group,
                          name=" ", description=s_description)
        # Create Security Group with group name longer than 255 chars
        s_name = 'securitygroup-'.ljust(260, '0')
        self.assertRaises(lib_exc.BadRequest,
                          self.client.create_security_group,
                          name=s_name, description=s_description)

    @decorators.skip_because(bug="1161411",
                             condition=CONF.service_available.neutron)
    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('777b6f14-aca9-4758-9e84-38783cfa58bc')
    def test_security_group_create_with_invalid_group_description(self):
        """Test creating security group with invalid group description

        Negative test: Security group should not be created with description
        longer than 255 chars. Empty description is allowed by the API
        reference, however.
        """
        s_name = data_utils.rand_name(
            prefix=CONF.resource_name_prefix, name='securitygroup')
        # Create Security Group with group description longer than 255 chars
        s_description = 'description-'.ljust(260, '0')
        self.assertRaises(lib_exc.BadRequest,
                          self.client.create_security_group,
                          name=s_name, description=s_description)

    @decorators.idempotent_id('9fdb4abc-6b66-4b27-b89c-eb215a956168')
    @testtools.skipIf(CONF.service_available.neutron,
                      "Neutron allows duplicate names for security groups")
    @decorators.attr(type=['negative'])
    def test_security_group_create_with_duplicate_name(self):
        """Test creating security group with duplicate name should fail"""
        s_name = data_utils.rand_name(
            prefix=CONF.resource_name_prefix, name='securitygroup')
        s_description = data_utils.rand_name(
            prefix=CONF.resource_name_prefix, name='description')
        self.create_security_group(name=s_name, description=s_description)
        # Now try the Security Group with the same 'Name'
        self.assertRaises(lib_exc.BadRequest,
                          self.client.create_security_group,
                          name=s_name, description=s_description)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('36a1629f-c6da-4a26-b8b8-55e7e5d5cd58')
    def test_delete_the_default_security_group(self):
        """Test deleting "default" security group should fail"""
        default_security_group_id = None
        body = self.client.list_security_groups()['security_groups']
        for i in range(len(body)):
            if body[i]['name'] == 'default':
                default_security_group_id = body[i]['id']
                break
        # Deleting the "default" Security Group
        self.assertRaises(lib_exc.BadRequest,
                          self.client.delete_security_group,
                          default_security_group_id)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('6727c00b-214c-4f9e-9a52-017ac3e98411')
    def test_delete_nonexistent_security_group(self):
        """Test deleting non existent security group should fail"""
        non_exist_id = self.generate_random_security_group_id()
        self.assertRaises(lib_exc.NotFound,
                          self.client.delete_security_group, non_exist_id)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('1438f330-8fa4-4aeb-8a94-37c250106d7f')
    def test_delete_security_group_without_passing_id(self):
        """Test deleting security group passing empty group id should fail"""
        self.assertRaises(lib_exc.NotFound,
                          self.client.delete_security_group, '')

    @decorators.idempotent_id('00579617-fe04-4e1c-9d08-ca7467d2e34b')
    @testtools.skipIf(CONF.service_available.neutron,
                      "Neutron does not check the security group ID")
    @decorators.attr(type=['negative'])
    def test_update_security_group_with_invalid_sg_id(self):
        """Test updating security group with invalid group id should fail"""
        s_name = data_utils.rand_name(
            prefix=CONF.resource_name_prefix, name='sg')
        s_description = data_utils.rand_name(
            prefix=CONF.resource_name_prefix, name='description')
        # Create a non int sg_id
        sg_id_invalid = data_utils.rand_name(
            prefix=CONF.resource_name_prefix, name='sg')
        self.assertRaises(lib_exc.BadRequest,
                          self.client.update_security_group, sg_id_invalid,
                          name=s_name, description=s_description)

    @decorators.idempotent_id('cda8d8b4-59f8-4087-821d-20cf5a03b3b1')
    @testtools.skipIf(CONF.service_available.neutron,
                      "Neutron does not check the security group name")
    @decorators.attr(type=['negative'])
    def test_update_security_group_with_invalid_sg_name(self):
        """Test updating security group to invalid group name should fail"""
        securitygroup = self.create_security_group()
        securitygroup_id = securitygroup['id']
        # Update Security Group with group name longer than 255 chars
        s_new_name = 'securitygroup-'.ljust(260, '0')
        self.assertRaises(lib_exc.BadRequest,
                          self.client.update_security_group,
                          securitygroup_id, name=s_new_name)

    @decorators.idempotent_id('97d12b1c-a610-4194-93f1-ba859e718b45')
    @testtools.skipIf(CONF.service_available.neutron,
                      "Neutron does not check the security group description")
    @decorators.attr(type=['negative'])
    def test_update_security_group_with_invalid_sg_des(self):
        """Test updating security group to invalid description should fail"""
        securitygroup = self.create_security_group()
        securitygroup_id = securitygroup['id']
        # Update Security Group with group description longer than 255 chars
        s_new_des = 'des-'.ljust(260, '0')
        self.assertRaises(lib_exc.BadRequest,
                          self.client.update_security_group,
                          securitygroup_id, description=s_new_des)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('27edee9c-873d-4da6-a68a-3c256efebe8f')
    def test_update_non_existent_security_group(self):
        """Test updating a non existent security group should fail"""
        non_exist_id = self.generate_random_security_group_id()
        s_name = data_utils.rand_name(
            prefix=CONF.resource_name_prefix, name='sg')
        s_description = data_utils.rand_name(
            prefix=CONF.resource_name_prefix, name='description')
        self.assertRaises(lib_exc.NotFound,
                          self.client.update_security_group,
                          non_exist_id, name=s_name,
                          description=s_description)



# Copyright 2016 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from urllib import parse as urllib

from oslo_serialization import jsonutils as json

from tempest.lib.common import rest_client


class UsersClient(rest_client.RestClient):
    api_version = "v3"

    def create_user(self, **kwargs):
        """Creates a user.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/identity/v3/#create-user
        """
        post_body = json.dumps({'user': kwargs})
        resp, body = self.post('users', post_body)
        self.expected_success(201, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def update_user(self, user_id, **kwargs):
        """Updates a user.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/identity/v3/#update-user
        """
        if 'id' not in kwargs:
            kwargs['id'] = user_id
        post_body = json.dumps({'user': kwargs})
        resp, body = self.patch('users/%s' % user_id, post_body)
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def update_user_password(self, user_id, **kwargs):
        """Update a user password

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/identity/v3/index.html#change-password-for-user
        """
        update_user = json.dumps({'user': kwargs})
        resp, _ = self.post('users/%s/password' % user_id, update_user)
        self.expected_success(204, resp.status)
        return rest_client.ResponseBody(resp)

    def list_user_projects(self, user_id, **params):
        """Lists the projects on which a user has roles assigned.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/identity/v3/#list-projects-for-user
        """
        url = 'users/%s/projects' % user_id
        if params:
            url += '?%s' % urllib.urlencode(params)
        resp, body = self.get(url)
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def list_users(self, **params):
        """Get the list of users.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/identity/v3/#list-users
        """
        url = 'users'
        if params:
            url += '?%s' % urllib.urlencode(params)
        resp, body = self.get(url)
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def show_user(self, user_id):
        """GET a user."""
        resp, body = self.get("users/%s" % user_id)
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def delete_user(self, user_id):
        """Deletes a User."""
        resp, body = self.delete("users/%s" % user_id)
        self.expected_success(204, resp.status)
        return rest_client.ResponseBody(resp, body)

    def list_user_groups(self, user_id, **params):
        """Lists groups which a user belongs to.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/identity/v3/#list-groups-to-which-a-user-belongs
        """
        url = 'users/%s/groups' % user_id
        if params:
            url += '?%s' % urllib.urlencode(params)
        resp, body = self.get(url)
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def create_user_ec2_credential(self, user_id, **kwargs):
        post_body = json.dumps(kwargs)
        resp, body = self.post('/users/%s/credentials/OS-EC2' % user_id,
                               post_body)
        self.expected_success(201, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def delete_user_ec2_credential(self, user_id, access):
        resp, body = self.delete('/users/%s/credentials/OS-EC2/%s' %
                                 (user_id, access))
        self.expected_success(204, resp.status)
        return rest_client.ResponseBody(resp, body)

    def list_user_ec2_credentials(self, user_id):
        resp, body = self.get('/users/%s/credentials/OS-EC2' % user_id)
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def show_user_ec2_credential(self, user_id, access):
        resp, body = self.get('/users/%s/credentials/OS-EC2/%s' %
                              (user_id, access))
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)



# Copyright 2013 NEC Corporation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.api.compute import base
from tempest.lib import decorators


class FlavorsAccessTestJSON(base.BaseV2ComputeAdminTest):
    """Tests Flavor Access API extension.

    Add and remove Flavor Access require admin privileges.
    """

    @classmethod
    def resource_setup(cls):
        super(FlavorsAccessTestJSON, cls).resource_setup()

        # Non admin tenant ID
        cls.tenant_id = cls.flavors_client.tenant_id
        cls.ram = 512
        cls.vcpus = 1
        cls.disk = 10

    @decorators.idempotent_id('ea2c2211-29fa-4db9-97c3-906d36fad3e0')
    def test_flavor_access_list_with_private_flavor(self):
        """Test listing flavor access for a private flavor

        Listing flavor access on a newly created private flavor will return
        an empty access list.
        """
        # Test to make sure that
        flavor = self.create_flavor(ram=self.ram, vcpus=self.vcpus,
                                    disk=self.disk, is_public='False')

        flavor_access = (self.admin_flavors_client.list_flavor_access(
                         flavor['id'])['flavor_access'])
        self.assertEmpty(flavor_access)

    @decorators.idempotent_id('59e622f6-bdf6-45e3-8ba8-fedad905a6b4')
    def test_flavor_access_add_remove(self):
        """Test add/remove flavor access to a given project"""
        flavor = self.create_flavor(ram=self.ram, vcpus=self.vcpus,
                                    disk=self.disk, is_public='False')

        # Add flavor access to a tenant.
        resp_body = {
            "tenant_id": str(self.tenant_id),
            "flavor_id": str(flavor['id']),
        }
        add_body = (self.admin_flavors_client.add_flavor_access(
            flavor['id'], self.tenant_id)['flavor_access'])
        self.assertIn(resp_body, add_body)

        # The flavor is present in list.
        flavors = self.flavors_client.list_flavors(detail=True)['flavors']
        self.assertIn(flavor['id'], map(lambda x: x['id'], flavors))

        # Remove flavor access from a tenant.
        remove_body = (self.admin_flavors_client.remove_flavor_access(
            flavor['id'], self.tenant_id)['flavor_access'])
        self.assertNotIn(resp_body, remove_body)

        # The flavor is not present in list.
        flavors = self.flavors_client.list_flavors(detail=True)['flavors']
        self.assertNotIn(flavor['id'], map(lambda x: x['id'], flavors))



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.lib import exceptions
from tempest.tests import base


class TestOutputParser(base.TestCase):

    def test_command_failed_exception(self):
        returncode = 1
        cmd = "foo"
        stdout = "output"
        stderr = "error"
        try:
            raise exceptions.CommandFailed(returncode, cmd, stdout, stderr)
        except exceptions.CommandFailed as e:
            self.assertIn(str(returncode), str(e))
            self.assertIn(cmd, str(e))
            self.assertIn(stdout, str(e))
            self.assertIn(stderr, str(e))



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import testtools
from testtools import matchers

from tempest.api.volume import base
from tempest.common import utils
from tempest.common import waiters
from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc

CONF = config.CONF


class VolumesSnapshotTestJSON(base.BaseVolumeTest):
    """Test volume snapshots"""

    create_default_network = True

    @classmethod
    def skip_checks(cls):
        super(VolumesSnapshotTestJSON, cls).skip_checks()
        if not CONF.volume_feature_enabled.snapshot:
            raise cls.skipException("Cinder volume snapshots are disabled")

    @classmethod
    def resource_setup(cls):
        super(VolumesSnapshotTestJSON, cls).resource_setup()
        cls.volume_origin = cls.create_volume()

    @decorators.idempotent_id('8567b54c-4455-446d-a1cf-651ddeaa3ff2')
    @utils.services('compute')
    def test_snapshot_create_delete_with_volume_in_use(self):
        """Test create/delete snapshot from volume attached to server"""
        # NOTE(zhufl) Here we create volume from self.image_ref for adding
        # coverage for "creating snapshot from non-blank volume".
        volume = self.create_volume(imageRef=self.image_ref)

        # Create a test instance
        server = self.create_server(wait_until='SSHABLE')

        # NOTE(danms): We are attaching this volume to a server, but we do
        # not need to block on detach during cleanup because we will be
        # deleting the server anyway.
        self.attach_volume(server['id'], volume['id'], wait_for_detach=False)

        # Snapshot a volume which attached to an instance with force=False
        self.assertRaises(lib_exc.BadRequest, self.create_snapshot,
                          volume['id'], force=False)

        # Snapshot a volume attached to an instance
        snapshot1 = self.create_snapshot(volume['id'], force=True)
        snapshot2 = self.create_snapshot(volume['id'], force=True)
        snapshot3 = self.create_snapshot(volume['id'], force=True)

        # Delete the snapshots. Some snapshot implementations can take
        # different paths according to order they are deleted.
        self.delete_snapshot(snapshot1['id'])
        self.delete_snapshot(snapshot3['id'])
        self.delete_snapshot(snapshot2['id'])

    @decorators.idempotent_id('5210a1de-85a0-11e6-bb21-641c676a5d61')
    @utils.services('compute')
    def test_snapshot_create_offline_delete_online(self):
        """Test creating snapshots when volume is detached and attached

        1. Create snapshot1 from volume1(not attached to any server)
        2. Attach volume1 to server1
        3. Create snapshot2 and snapshot3 from volume1
        4. Delete snapshot3, snapshot1, snapshot2
        """
        # Create a snapshot while it is not attached
        snapshot1 = self.create_snapshot(self.volume_origin['id'])

        # Create a server and attach it
        server = self.create_server(wait_until='SSHABLE')
        # NOTE(danms): We are attaching this volume to a server, but we do
        # not need to block on detach during cleanup because we will be
        # deleting the server anyway.
        self.attach_volume(server['id'], self.volume_origin['id'],
                           wait_for_detach=False)

        # Now that the volume is attached, create other snapshots
        snapshot2 = self.create_snapshot(self.volume_origin['id'], force=True)
        snapshot3 = self.create_snapshot(self.volume_origin['id'], force=True)

        # Delete the snapshots. Some snapshot implementations can take
        # different paths according to order they are deleted.
        self.delete_snapshot(snapshot3['id'])
        self.delete_snapshot(snapshot1['id'])
        self.delete_snapshot(snapshot2['id'])

    @decorators.idempotent_id('2a8abbe4-d871-46db-b049-c41f5af8216e')
    def test_snapshot_create_get_list_update_delete(self):
        """Test create/get/list/update/delete snapshot"""
        # Create a snapshot with metadata
        metadata = {"snap-meta1": "value1",
                    "snap-meta2": "value2",
                    "snap-meta3": "value3"}
        snapshot = self.create_snapshot(self.volume_origin['id'],
                                        metadata=metadata)

        # Get the snap and check for some of its details
        snap_get = self.snapshots_client.show_snapshot(
            snapshot['id'])['snapshot']
        self.assertEqual(self.volume_origin['id'],
                         snap_get['volume_id'],
                         "Referred volume origin mismatch")
        self.assertEqual(self.volume_origin['size'], snap_get['size'])

        # Verify snapshot metadata
        self.assertThat(snap_get['metadata'].items(),
                        matchers.ContainsAll(metadata.items()))

        # Compare also with the output from the list action
        tracking_data = (snapshot['id'], snapshot['name'])
        snaps_list = self.snapshots_client.list_snapshots()['snapshots']
        snaps_data = [(f['id'], f['name']) for f in snaps_list]
        self.assertIn(tracking_data, snaps_data)

        # Updates snapshot with new values
        new_s_name = data_utils.rand_name(
            prefix=CONF.resource_name_prefix,
            name=self.__class__.__name__ + '-new-snap')
        new_desc = 'This is the new description of snapshot.'
        params = {'name': new_s_name,
                  'description': new_desc}
        update_snapshot = self.snapshots_client.update_snapshot(
            snapshot['id'], **params)['snapshot']
        # Assert response body for update_snapshot method
        self.assertEqual(new_s_name, update_snapshot['name'])
        self.assertEqual(new_desc, update_snapshot['description'])
        # Assert response body for show_snapshot method
        updated_snapshot = self.snapshots_client.show_snapshot(
            snapshot['id'])['snapshot']
        self.assertEqual(new_s_name, updated_snapshot['name'])
        self.assertEqual(new_desc, updated_snapshot['description'])

        # Delete the snapshot
        self.delete_snapshot(snapshot['id'])

    def _create_volume_from_snapshot(self, extra_size=0):
        src_size = CONF.volume.volume_size
        size = src_size + extra_size

        src_vol = self.create_volume(size=src_size)
        src_snap = self.create_snapshot(src_vol['id'])

        dst_vol = self.create_volume(snapshot_id=src_snap['id'],
                                     size=size)
        # NOTE(zhufl): dst_vol is created based on snapshot, so dst_vol
        # should be deleted before deleting snapshot, otherwise deleting
        # snapshot will end with status 'error-deleting'. This depends on
        # the implementation mechanism of vendors, generally speaking,
        # some verdors will use "virtual disk clone" which will promote
        # disk clone speed, and in this situation the "disk clone"
        # is just a relationship between volume and snapshot.
        self.addCleanup(self.delete_volume, self.volumes_client, dst_vol['id'])

        volume = self.volumes_client.show_volume(dst_vol['id'])['volume']
        # Should allow
        self.assertEqual(volume['snapshot_id'], src_snap['id'])
        self.assertEqual(volume['size'], size)

    @decorators.idempotent_id('677863d1-3142-456d-b6ac-9924f667a7f4')
    def test_volume_from_snapshot(self):
        """Test creating volume from snapshot with extending size"""
        self._create_volume_from_snapshot(
            extra_size=CONF.volume.volume_size_extend)

    @decorators.idempotent_id('053d8870-8282-4fff-9dbb-99cb58bb5e0a')
    def test_volume_from_snapshot_no_size(self):
        """Test creating volume from snapshot with original size"""
        self._create_volume_from_snapshot()

    @decorators.idempotent_id('bbcfa285-af7f-479e-8c1a-8c34fc16543c')
    @testtools.skipUnless(CONF.volume_feature_enabled.backup,
                          "Cinder backup is disabled")
    def test_snapshot_backup(self):
        """Test creating backup from snapshot and volume

        1. Create snapshot1 from volume1
        2. Create backup from volume1 and snapshot1
        3. Check the created backup's volume is volume1 and snapshot
           is snapshot1
        """
        # Create a snapshot
        snapshot = self.create_snapshot(volume_id=self.volume_origin['id'])

        backup = self.create_backup(volume_id=self.volume_origin['id'],
                                    snapshot_id=snapshot['id'])
        waiters.wait_for_volume_resource_status(self.snapshots_client,
                                                snapshot['id'], 'available')
        backup_info = self.backups_client.show_backup(backup['id'])['backup']
        self.assertEqual(self.volume_origin['id'], backup_info['volume_id'])
        self.assertEqual(snapshot['id'], backup_info['snapshot_id'])



# Copyright 2014 NEC Corporation.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

common_agent_info = {
    'type': 'object',
    'properties': {
        'agent_id': {'type': ['integer', 'string']},
        'hypervisor': {'type': 'string'},
        'os': {'type': 'string'},
        'architecture': {'type': 'string'},
        'version': {'type': 'string'},
        'url': {'type': 'string', 'format': 'uri'},
        'md5hash': {'type': 'string'}
    },
    'additionalProperties': False,
    'required': ['agent_id', 'hypervisor', 'os', 'architecture',
                 'version', 'url', 'md5hash']
}

list_agents = {
    'status_code': [200],
    'response_body': {
        'type': 'object',
        'properties': {
            'agents': {
                'type': 'array',
                'items': common_agent_info
            }
        },
        'additionalProperties': False,
        'required': ['agents']
    }
}

create_agent = {
    'status_code': [200],
    'response_body': {
        'type': 'object',
        'properties': {
            'agent': common_agent_info
        },
        'additionalProperties': False,
        'required': ['agent']
    }
}

update_agent = {
    'status_code': [200],
    'response_body': {
        'type': 'object',
        'properties': {
            'agent': {
                'type': 'object',
                'properties': {
                    'agent_id': {'type': ['integer', 'string']},
                    'version': {'type': 'string'},
                    'url': {'type': 'string', 'format': 'uri'},
                    'md5hash': {'type': 'string'}
                },
                'additionalProperties': False,
                'required': ['agent_id', 'version', 'url', 'md5hash']
            }
        },
        'additionalProperties': False,
        'required': ['agent']
    }
}

delete_agent = {
    'status_code': [200]
}
